@@ -1,7 +1,6 @@
 /****************************************************************************
-** $Id: qt/aclock.h   3.2.3   edited May 13 09:08 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -14,30 +13,31 @@
 #include <qwidget.h>
 #include <qdatetime.h>
 
-
+class QTimer;
 class AnalogClock : public QWidget		// analog clock widget
 {
     Q_OBJECT
 public:
     AnalogClock( QWidget *parent=0, const char *name=0 );
     void setAutoMask(bool b);
-
-public slots:    
-    void setTime( const QTime & t );
     
 protected:
     void updateMask();
     void paintEvent( QPaintEvent *);
     void mousePressEvent( QMouseEvent *);
     void mouseMoveEvent( QMouseEvent *);
+    void drawClock( QPainter* );
     
 private slots:
-    void drawClock( QPainter* );
-    void	timeout();
+    void timeout();
+
+public slots:
+    void setTime( const QTime & t );
 
 private:
     QPoint clickPos;
-    QTime	time;
+    QTime time;
+    QTimer *internalTimer;
 };
 
 
