@@ -1,37 +1,40 @@
 /****************************************************************************
-** $Id: qt/qfontdata_p.h   3.3.4   edited Dec 16 11:33 $
 **
 ** Definition of internal QFontData struct
 **
 ** Created : 941229
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
-** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
-** LICENSE.QPL included in the packaging of this file.
+** This file may be used under the terms of the GNU General
+** Public License versions 2.0 or 3.0 as published by the Free
+** Software Foundation and appearing in the files LICENSE.GPL2
+** and LICENSE.GPL3 included in the packaging of this file.
+** Alternatively you may (at your option) use any later version
+** of the GNU General Public License if such license has been
+** publicly approved by Trolltech ASA (or its successors, if any)
+** and the KDE Free Qt Foundation.
 **
-** This file may be distributed and/or modified under the terms of the
-** GNU General Public License version 2 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/.
+** If you are unsure which license is appropriate for your use, please
+** review the following information:
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
 **
-** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
-** licenses may use this file in accordance with the Qt Commercial License
-** Agreement provided with the Software.
+** This file may be used under the terms of the Q Public License as
+** defined by Trolltech ASA and appearing in the file LICENSE.QPL
+** included in the packaging of this file.  Licensees holding valid Qt
+** Commercial licenses may use this file in accordance with the Qt
+** Commercial License Agreement provided with the Software.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-**
-** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
-**   information about Qt Commercial License Agreements.
-** See http://www.trolltech.com/qpl/ for QPL licensing information.
-** See http://www.trolltech.com/gpl/ for GPL licensing information.
-**
-** Contact info@trolltech.com if any conditions of this licensing are
-** not clear to you.
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
+** herein.
 **
 **********************************************************************/
 
@@ -52,6 +55,7 @@
 
 #include "qobject.h"
 #include "qfont.h"
+#include "qpaintdevicemetrics.h"
 
 // forwards
 class QFontEngine;
@@ -203,27 +207,33 @@ public:
     // universal key structure.  QFontEngineDatas and QFontEngines are cached using
     // the same keys
     struct Key {
-	Key() : screen( 0 ) { }
-	Key( const QFontDef &d, QFont::Script c, int s = 0 )
-#if !defined (Q_CC_HPACC)
-            : def( d ), script( c ), screen( s ) { }
+	Key() : script(0), screen( 0 ), dpi(0) { }
+	Key( const QFontDef &d, QFont::Script c, int s, QPaintDevice *pdev )
+            : script(c), screen(s) {
+            def = d;
+#ifdef Q_WS_X11
+            dpi = pdev ? QPaintDeviceMetrics(pdev).logicalDpiY() : 0;
 #else
-            // work around an aCC compiler bug
-            : script(c), screen(s) { def = d; }
+            Q_UNUSED(pdev);
+            dpi = 0;
 #endif
+        }
 
 	QFontDef def;
 	int script;
 	int screen;
+        int dpi;
 
 	inline bool operator<( const Key &other ) const
 	{
 	    if ( script != other.script ) return script < other.script;
 	    if ( screen != other.screen ) return screen < other.screen;
+	    if ( dpi != other.dpi ) return dpi < other.dpi;
 	    return def < other.def;
 	}
 	inline bool operator==( const Key &other ) const
-	{ return def == other.def && script == other.script && screen == other.screen; }
+	{ return def == other.def && script == other.script &&
+                screen == other.screen && dpi == other.dpi; }
     };
 
     // QFontEngineData cache
