@@ -4,33 +4,37 @@
 **
 ** Created : 000607
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the table module of the Qt GUI Toolkit.
 **
-** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
-** LICENSE.QPL included in the packaging of this file.
+** This file may be used under the terms of the GNU General
+** Public License versions 2.0 or 3.0 as published by the Free
+** Software Foundation and appearing in the files LICENSE.GPL2
+** and LICENSE.GPL3 included in the packaging of this file.
+** Alternatively you may (at your option) use any later version
+** of the GNU General Public License if such license has been
+** publicly approved by Trolltech ASA (or its successors, if any)
+** and the KDE Free Qt Foundation.
 **
-** This file may be distributed and/or modified under the terms of the
-** GNU General Public License version 2 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/.
+** If you are unsure which license is appropriate for your use, please
+** review the following information:
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
 **
-** Licensees holding valid Qt Enterprise Edition licenses may use this
-** file in accordance with the Qt Commercial License Agreement provided
-** with the Software.
+** This file may be used under the terms of the Q Public License as
+** defined by Trolltech ASA and appearing in the file LICENSE.QPL
+** included in the packaging of this file.  Licensees holding valid Qt
+** Commercial licenses may use this file in accordance with the Qt
+** Commercial License Agreement provided with the Software.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-**
-** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
-**   information about Qt Commercial License Agreements.
-** See http://www.trolltech.com/qpl/ for QPL licensing information.
-** See http://www.trolltech.com/gpl/ for GPL licensing information.
-**
-** Contact info@trolltech.com if any conditions of this licensing are
-** not clear to you.
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
+** herein.
 **
 **********************************************************************/
 
@@ -38,21 +42,31 @@
 #define QTABLE_H
 
 #ifndef QT_H
-#include <qscrollview.h>
-#include <qpixmap.h>
-#include <qptrvector.h>
-#include <qheader.h>
-#include <qmemarray.h>
-#include <qptrlist.h>
-#include <qguardedptr.h>
-#include <qshared.h>
-#include <qintdict.h>
-#include <qstringlist.h>
+#include "qscrollview.h"
+#include "qpixmap.h"
+#include "qptrvector.h"
+#include "qheader.h"
+#include "qmemarray.h"
+#include "qptrlist.h"
+#include "qguardedptr.h"
+#include "qshared.h"
+#include "qintdict.h"
+#include "qstringlist.h"
 #endif // QT_H
 
 
 #ifndef QT_NO_TABLE
 
+#if !defined( QT_MODULE_TABLE ) || defined( QT_LICENSE_PROFESSIONAL ) || defined( QT_INTERNAL_TABLE )
+#define QM_EXPORT_TABLE
+#ifndef QM_TEMPLATE_EXTERN_TABLE
+#   define QM_TEMPLATE_EXTERN_TABLE
+#endif
+#else
+#define QM_EXPORT_TABLE Q_EXPORT
+#define QM_TEMPLATE_EXTERN_TABLE Q_TEMPLATE_EXTERN
+#endif
+
 class QTableHeader;
 class QValidator;
 class QTable;
@@ -67,10 +81,11 @@ struct QTablePrivate;
 struct QTableHeaderPrivate;
 
 
-class Q_EXPORT QTableSelection
+class QM_EXPORT_TABLE QTableSelection
 {
 public:
     QTableSelection();
+    QTableSelection( int start_row, int start_col, int end_row, int end_col );
     void init( int row, int col );
     void expandTo( int row, int col );
     bool operator==( const QTableSelection &s ) const;
@@ -82,8 +97,11 @@ public:
     int rightCol() const { return rCol; }
     int anchorRow() const { return aRow; }
     int anchorCol() const { return aCol; }
+    int numRows() const;
+    int numCols() const;
 
     bool isActive() const { return active; }
+    bool isEmpty() const { return numRows() == 0; }
 
 private:
     uint active : 1;
@@ -92,14 +110,17 @@ private:
     int aRow, aCol;
 };
 
+#define Q_DEFINED_QTABLE_SELECTION
+#include "qwinexport.h"
 
-class Q_EXPORT QTableItem : public Qt
+class QM_EXPORT_TABLE QTableItem : public Qt
 {
     friend class QTable;
 
 public:
     enum EditType { Never, OnTyping, WhenCurrent, Always };
 
+    QTableItem( QTable *table, EditType et );
     QTableItem( QTable *table, EditType et, const QString &text );
     QTableItem( QTable *table, EditType et, const QString &text,
                 const QPixmap &p );
@@ -154,13 +175,19 @@ private:
     uint enabled : 1;
     int rw, cl;
     int rowspan, colspan;
-
+#if (QT_VERSION >= 0x040000)
+#error "Add a setAlignment() function in 4.0 (but no d pointer)"
+#endif
 };
 
-class Q_EXPORT QComboTableItem : public QTableItem
+#define Q_DEFINED_QTABLE_ITEM
+#include "qwinexport.h"
+
+class QM_EXPORT_TABLE QComboTableItem : public QTableItem
 {
 public:
     QComboTableItem( QTable *table, const QStringList &list, bool editable = FALSE );
+    ~QComboTableItem();
     virtual QWidget *createEditor() const;
     virtual void setContentFromEditor( QWidget *w );
     virtual void paint( QPainter *p, const QColorGroup &cg,
@@ -170,6 +197,9 @@ public:
     int currentItem() const;
     QString currentText() const;
     int count() const;
+#if !defined(Q_NO_USING_KEYWORD)
+    using QTableItem::text;
+#endif
     QString text( int i ) const;
     virtual void setEditable( bool b );
     bool isEditable() const;
@@ -178,19 +208,24 @@ public:
     int rtti() const;
     static int RTTI;
 
+    QSize sizeHint() const;
+
 private:
     QComboBox *cb;
     QStringList entries;
     int current;
     bool edit;
     static QComboBox *fakeCombo;
+    static QWidget *fakeComboWidget;
+    static int fakeRef;
 
 };
 
-class Q_EXPORT QCheckTableItem : public QTableItem
+class QM_EXPORT_TABLE QCheckTableItem : public QTableItem
 {
 public:
     QCheckTableItem( QTable *table, const QString &txt );
+    void setText( const QString &t );
     virtual QWidget *createEditor() const;
     virtual void setContentFromEditor( QWidget *w );
     virtual void paint( QPainter *p, const QColorGroup &cg,
@@ -201,23 +236,15 @@ public:
     int rtti() const;
     static int RTTI;
 
+    QSize sizeHint() const;
+
 private:
     QCheckBox *cb;
     bool checked;
 
 };
 
-#if defined(Q_TEMPLATEDLL)
-// MOC_SKIP_BEGIN
-template class Q_EXPORT QPtrVector<QTableItem>;
-template class Q_EXPORT QPtrVector<QWidget>;
-template class Q_EXPORT QPtrList<QTableSelection>;
-template class Q_EXPORT QIntDict<int>;
-// MOC_SKIP_END
-#endif
-
-
-class Q_EXPORT QTable : public QScrollView
+class QM_EXPORT_TABLE QTable : public QScrollView
 {
     Q_OBJECT
     Q_ENUMS( SelectionMode FocusStyle )
@@ -230,6 +257,7 @@ class Q_EXPORT QTable : public QScrollView
     Q_PROPERTY( bool sorting READ sorting WRITE setSorting )
     Q_PROPERTY( SelectionMode selectionMode READ selectionMode WRITE setSelectionMode )
     Q_PROPERTY( FocusStyle focusStyle READ focusStyle WRITE setFocusStyle )
+    Q_PROPERTY( int numSelections READ numSelections )
 
     friend class QTableHeader;
     friend class QComboTableItem;
@@ -286,6 +314,10 @@ public:
     virtual void removeSelection( int num );
     virtual int currentSelection() const;
 
+    void selectCells( int start_row, int start_col, int end_row, int end_col );
+    void selectRow( int row );
+    void selectColumn( int col );
+
     bool showGrid() const;
 
     bool columnMovingEnabled() const;
@@ -322,6 +354,8 @@ public:
     virtual void setFocusStyle( FocusStyle fs );
     FocusStyle focusStyle() const;
 
+    void updateHeaderStates();
+
 public slots:
     virtual void setNumRows( int r );
     virtual void setNumCols( int r );
@@ -330,6 +364,8 @@ public slots:
     virtual void hideColumn( int col );
     virtual void showRow( int row );
     virtual void showColumn( int col );
+    bool isRowHidden( int row ) const;
+    bool isColumnHidden( int col ) const;
 
     virtual void setColumnWidth( int col, int w );
     virtual void setRowHeight( int row, int h );
@@ -369,6 +405,9 @@ public slots:
 
     virtual void editCell( int row, int col, bool replace = FALSE );
 
+    void setRowLabels( const QStringList &labels );
+    void setColumnLabels( const QStringList &labels );
+
 protected:
     enum EditMode { NotEditing, Editing, Replacing };
     void drawContents( QPainter *p, int cx, int cy, int cw, int ch );
@@ -382,6 +421,7 @@ protected:
     void focusOutEvent( QFocusEvent* );
     void viewportResizeEvent( QResizeEvent * );
     void showEvent( QShowEvent *e );
+    void paintEvent( QPaintEvent *e );
     void setEditMode( EditMode mode, int row, int col );
 #ifndef QT_NO_DRAGANDDROP
     virtual void contentsDragEnterEvent( QDragEnterEvent *e );
@@ -431,6 +471,7 @@ signals:
 private slots:
     void doAutoScroll();
     void doValueChanged();
+    void updateGeometriesSlot();
 
 private:
     void contentsMousePressEventEx( QMouseEvent* );
@@ -456,8 +497,23 @@ private:
     void updateRowWidgets( int row );
     void updateColWidgets( int col );
     bool isSelected( int row, int col, bool includeCurrent ) const;
-    void setCurrentCell( int row, int col, bool updateSelections );
+    void setCurrentCell( int row, int col, bool updateSelections, bool ensureVisible = FALSE );
     void fixCell( int &row, int &col, int key );
+    void delayedUpdateGeometries();
+    struct TableWidget
+    {
+	TableWidget( QWidget *w, int r, int c ) : wid( w ), row( r ), col ( c ) {}
+	QWidget *wid;
+	int row, col;
+    };
+    void saveContents( QPtrVector<QTableItem> &tmp,
+		       QPtrVector<TableWidget> &tmp2 );
+    void updateHeaderAndResizeContents( QTableHeader *header,
+					int num, int colRow,
+					int width, bool &updateBefore );
+    void restoreContents( QPtrVector<QTableItem> &tmp,
+			  QPtrVector<TableWidget> &tmp2 );
+    void finishContentsResze( bool updateBefore );
 
 private:
     QPtrVector<QTableItem> contents;
@@ -476,7 +532,7 @@ private:
     bool mCols : 1;
     bool asc : 1;
     bool doSort : 1;
-    bool mousePressed : 1;
+    bool unused : 1;
     bool readOnly : 1;
     bool shouldClearSelection : 1;
     bool dEnabled : 1;
@@ -492,11 +548,17 @@ private:
     int startDragCol;
     QPoint dragStartPos;
     int oldCurrentRow, oldCurrentCol;
-    QWidget *topLeftCorner;
+    QWidget *unused_topLeftCorner; //### remove in 4.0
     FocusStyle focusStl;
-    QSize cachedSizeHint;
+    QSize unused_cachedSizeHint; // ### remove in 4.0
 
+#if defined(Q_DISABLE_COPY)
+    QTable( const QTable & );
+    QTable &operator=( const QTable & );
+#endif
 };
 
+#define Q_DEFINED_QTABLE
+#include "qwinexport.h"
 #endif // QT_NO_TABLE
 #endif // TABLE_H
