@@ -1,16 +1,16 @@
 /****************************************************************************
-** $Id: qt/qmessagebox.cpp   3.3.5   edited Aug 31 12:13 $
+** $Id: qt/qmessagebox.cpp   3.3.8   edited Jan 11 14:37 $
 **
 ** Implementation of QMessageBox class
 **
 ** Created : 950503
 **
-** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the dialogs module of the Qt GUI Toolkit.
 **
 ** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
+** as defined by Trolltech ASA of Norway and appearing in the file
 ** LICENSE.QPL included in the packaging of this file.
 **
 ** This file may be distributed and/or modified under the terms of the
@@ -527,12 +527,13 @@ QMessageBox::QMessageBox( QWidget *parent, const char *name )
     Example:
     \code
     QMessageBox mb( "Application Name",
-	"Hardware failure.\n\nDisk error detected\nDo you want to stop?",
-	QMessageBox::Question,
-	QMessageBox::Yes | QMessageBox::Default,
-	QMessageBox::No  | QMessageBox::Escape );
+        "Hardware failure.\n\nDisk error detected\nDo you want to stop?",
+        QMessageBox::Question,
+        QMessageBox::Yes | QMessageBox::Default,
+        QMessageBox::No  | QMessageBox::Escape,
+        QMessageBox::NoButton );
     if ( mb.exec() == QMessageBox::No )
-	// try again
+        // try again
     \endcode
 
     If \a parent is 0, the message box becomes an application-global
