@@ -1,37 +1,40 @@
 /****************************************************************************
-** $Id: qt/qasyncimageio.cpp   3.3.4   edited Dec 14 11:56 $
 **
 ** Implementation of asynchronous image/movie loading classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
-** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
-** LICENSE.QPL included in the packaging of this file.
+** This file may be used under the terms of the GNU General
+** Public License versions 2.0 or 3.0 as published by the Free
+** Software Foundation and appearing in the files LICENSE.GPL2
+** and LICENSE.GPL3 included in the packaging of this file.
+** Alternatively you may (at your option) use any later version
+** of the GNU General Public License if such license has been
+** publicly approved by Trolltech ASA (or its successors, if any)
+** and the KDE Free Qt Foundation.
 **
-** This file may be distributed and/or modified under the terms of the
-** GNU General Public License version 2 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/.
+** If you are unsure which license is appropriate for your use, please
+** review the following information:
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
 **
-** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
-** licenses may use this file in accordance with the Qt Commercial License
-** Agreement provided with the Software.
+** This file may be used under the terms of the Q Public License as
+** defined by Trolltech ASA and appearing in the file LICENSE.QPL
+** included in the packaging of this file.  Licensees holding valid Qt
+** Commercial licenses may use this file in accordance with the Qt
+** Commercial License Agreement provided with the Software.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-**
-** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
-**   information about Qt Commercial License Agreements.
-** See http://www.trolltech.com/qpl/ for QPL licensing information.
-** See http://www.trolltech.com/gpl/ for GPL licensing information.
-**
-** Contact info@trolltech.com if any conditions of this licensing are
-** not clear to you.
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
+** herein.
 **
 **********************************************************************/
 
@@ -245,6 +248,7 @@ private:
     int lzwsize;
     bool lcmap;
     int swidth, sheight;
+    int width, height;
     int left, top, right, bottom;
     enum Disposal { NoDisposal, DoNotChange, RestoreBackground, RestoreImage };
     Disposal disposal;
@@ -884,20 +888,20 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 	    if (count==10) {
 		int newleft=LM(hold[1], hold[2]);
 		int newtop=LM(hold[3], hold[4]);
-		int width=LM(hold[5], hold[6]);
-		int height=LM(hold[7], hold[8]);
+		int newwidth=LM(hold[5], hold[6]);
+		int newheight=LM(hold[7], hold[8]);
 
 		// disbelieve ridiculous logical screen sizes,
 		// unless the image frames are also large.
-		if ( swidth/10 > QMAX(width,200) )
+		if ( swidth/10 > QMAX(newwidth,200) )
 		    swidth = -1;
-		if ( sheight/10 > QMAX(height,200) )
+		if ( sheight/10 > QMAX(newheight,200) )
 		    sheight = -1;
 
 		if ( swidth <= 0 )
-		    swidth = newleft + width;
+		    swidth = newleft + newwidth;
 		if ( sheight <= 0 )
-		    sheight = newtop + height;
+		    sheight = newtop + newheight;
 
 		if (img.isNull()) {
 		    img.create(swidth, sheight, 32);
@@ -912,25 +916,11 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 
 		left = newleft;
 		top = newtop;
+		width = newwidth;
+		height = newheight;
 
-		// Sanity check frame size - must fit on "screen".
-		if (left >= swidth) left=QMAX(0, swidth-1);
-		if (top >= sheight) top=QMAX(0, sheight-1);
-		if (left+width >= swidth) {
-		    if ( width <= swidth )
-			left=swidth-width;
-		    else
-			width=swidth-left;
-		}
-		if (top+height >= sheight) {
-		    if ( height <= sheight )
-			top=sheight-height;
-		    else
-			height=sheight-top;
-		}
-
-		right=QMAX( 0, left+width-1);
-		bottom=QMAX(0, top+height-1);
+		right=QMAX( 0, QMIN(left+width, swidth)-1);
+		bottom=QMAX(0, QMIN(top+height, sheight)-1);
 		lcmap=!!(hold[9]&0x80);
 		interlace=!!(hold[9]&0x40);
 		//bool lcmsortflag=!!(hold[9]&0x20);
@@ -946,7 +936,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		}
 		frame++;
 		if ( frame == 0 ) {
-		    if ( left || top || width!=swidth || height!=sheight ) {
+		    if ( left || top || width<swidth || height<sheight ) {
 			// Not full-size image - erase with bg or transparent
 			if ( trans_index >= 0 ) {
 			    fillRect(img, 0, 0, swidth, sheight, color(trans_index));
@@ -994,7 +984,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		sp = stack;
                 firstcode = oldcode = 0;
 		needfirst = FALSE;
-		out_of_bounds = FALSE;
+		out_of_bounds = left>=swidth || y>=sheight;
 	    }
 	    break;
 	  case TableImageLZWSize: {
@@ -1067,10 +1057,9 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 			x++;
 			if (x>=swidth) out_of_bounds = TRUE;
 			needfirst=FALSE;
-			if (x>right) {
+			if (x>=left+width) {
 			    x=left;
-			    if (out_of_bounds)
-				out_of_bounds = left>=swidth || y>=sheight;
+                            out_of_bounds = left>=swidth || y>=sheight;
 			    nextY(img,consumer);
 			}
 		    } else {
@@ -1111,10 +1100,9 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 				line[y][x] = color(*sp);
 			    x++;
 			    if (x>=swidth) out_of_bounds = TRUE;
-			    if (x>right) {
+			    if (x>=left+width) {
 				x=left;
-				if (out_of_bounds)
-				    out_of_bounds = left>=swidth || y>=sheight;
+                                out_of_bounds = left>=swidth || y>=sheight;
 				nextY(img,consumer);
 			    }
 			}
