@@ -1,37 +1,40 @@
 /****************************************************************************
-** $Id: qt/qfont.cpp   3.2.3   edited Oct 27 17:07 $
 **
 ** Implementation of QFont, QFontMetrics and QFontInfo classes
 **
 ** Created : 941207
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
-** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
-** LICENSE.QPL included in the packaging of this file.
+** This file may be used under the terms of the GNU General
+** Public License versions 2.0 or 3.0 as published by the Free
+** Software Foundation and appearing in the files LICENSE.GPL2
+** and LICENSE.GPL3 included in the packaging of this file.
+** Alternatively you may (at your option) use any later version
+** of the GNU General Public License if such license has been
+** publicly approved by Trolltech ASA (or its successors, if any)
+** and the KDE Free Qt Foundation.
 **
-** This file may be distributed and/or modified under the terms of the
-** GNU General Public License version 2 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/.
+** If you are unsure which license is appropriate for your use, please
+** review the following information:
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
 **
-** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
-** licenses may use this file in accordance with the Qt Commercial License
-** Agreement provided with the Software.
+** This file may be used under the terms of the Q Public License as
+** defined by Trolltech ASA and appearing in the file LICENSE.QPL
+** included in the packaging of this file.  Licensees holding valid Qt
+** Commercial licenses may use this file in accordance with the Qt
+** Commercial License Agreement provided with the Software.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-**
-** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
-**   information about Qt Commercial License Agreements.
-** See http://www.trolltech.com/qpl/ for QPL licensing information.
-** See http://www.trolltech.com/gpl/ for GPL licensing information.
-**
-** Contact info@trolltech.com if any conditions of this licensing are
-** not clear to you.
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
+** herein.
 **
 **********************************************************************/
 
@@ -88,16 +91,17 @@ bool QFontDef::operator==( const QFontDef &other ) const
       us to compare e.g. "Helvetica" and "Helvetica [Adobe]" with
       positive results.
     */
-    QString this_family, this_foundry, other_family, other_foundry;
-    QFontDatabase::parseFontName(family, this_foundry, this_family);
-    QFontDatabase::parseFontName(other.family, other_foundry, other_family);
-
-    if ((pointSize == -1 || other.pointSize == -1) && pixelSize != other.pixelSize)
-	return FALSE;
-    if ((pixelSize == -1 || other.pixelSize == -1) && pointSize != other.pointSize &&
-	(QABS(pointSize - other.pointSize) >= 5 ||
-	 qRound(pointSize/10.) != qRound(other.pointSize/10.)))
+    if (pixelSize != -1 && other.pixelSize != -1) {
+	if (pixelSize != other.pixelSize)
+	    return FALSE;
+    } else if (pointSize != -1 && other.pointSize != -1) {
+	if (pointSize != other.pointSize
+	    && (QABS(pointSize - other.pointSize) >= 5
+		|| qRound(pointSize/10.) != qRound(other.pointSize/10.)))
+	    return FALSE;
+    } else {
 	return FALSE;
+    }
 
     if (!ignorePitch && !other.ignorePitch && fixedPitch != other.fixedPitch)
 	return FALSE;
@@ -105,6 +109,10 @@ bool QFontDef::operator==( const QFontDef &other ) const
     if (stretch != 0 && other.stretch != 0 && stretch != other.stretch)
 	return FALSE;
 
+    QString this_family, this_foundry, other_family, other_foundry;
+    QFontDatabase::parseFontName(family, this_foundry, this_family);
+    QFontDatabase::parseFontName(other.family, other_foundry, other_family);
+
     return ( styleHint     == other.styleHint
 	    && styleStrategy == other.styleStrategy
 	    && weight        == other.weight
@@ -513,6 +521,9 @@ QFontEngineData::~QFontEngineData()
 
     \value KatakanaHalfWidth
 
+    \value Limbu (Unicode 4.0)
+    \value TaiLe (Unicode 4.0)
+
     \value Unicode includes all the above scripts.
 */
 
@@ -1075,16 +1086,14 @@ QFont::StyleHint QFont::styleHint() const
     \value PreferDefault the default style strategy. It does not prefer
 	   any type of font.
     \value PreferBitmap prefers bitmap fonts (as opposed to outline
-	   fonts). On X11, this will cause Qt to disregard fonts from
-	   the Xft font extension.
+	   fonts).
     \value PreferDevice prefers device fonts.
     \value PreferOutline prefers outline fonts (as opposed to bitmap fonts).
     \value ForceOutline forces the use of outline fonts.
     \value NoAntialias don't antialias the fonts.
     \value PreferAntialias antialias if possible.
     \value OpenGLCompatible forces the use of OpenGL compatible
-           fonts. On X11, this will cause Qt to disregard fonts from
-           the Xft font extension.
+           fonts.
 
     Any of these may be OR-ed with one of these flags:
 
@@ -1633,7 +1642,7 @@ bool QFont::fromString(const QString &descrip)
 #ifndef QT_NO_STRINGLIST
     QStringList l(QStringList::split(',', descrip));
 
-    int count = l.count();
+    int count = (int)l.count();
 #else
     int count = 0;
     QString l[11];
@@ -1649,7 +1658,8 @@ bool QFont::fromString(const QString &descrip)
     if ( !count || ( count > 2 && count < 9 ) || count > 11 ) {
 
 #ifdef QT_CHECK_STATE
-	qWarning("QFont::fromString: invalid description '%s'", descrip.latin1());
+	qWarning("QFont::fromString: invalid description '%s'",
+                 descrip.isEmpty() ? "(empty)" : descrip.latin1());
 #endif
 
 	return FALSE;
@@ -1820,7 +1830,7 @@ QDataStream &operator>>( QDataStream &s, QFont &font )
     updated.
 
     \i QPainter::fontMetrics() returns the font metrics for a
-    painter's current font. If the widget's font is changed later, the
+    painter's current font. If the painter's font is changed later, the
     font metrics object is \e not updated.
     \endlist
 
@@ -2246,27 +2256,28 @@ int QFontMetrics::width( const QString &str, int len ) const
 	unsigned short uc = ch->unicode();
 	if (uc < QFontEngineData::widthCacheSize && d->engineData && d->engineData->widthCache[uc])
 	    width += d->engineData->widthCache[uc];
-	else if ( ::category( *ch ) != QChar::Mark_NonSpacing ) {
+	else {
 	    QFont::Script script;
 	    SCRIPT_FOR_CHAR( script, *ch );
 
 	    if (script >= QFont::Arabic && script <= QFont::Khmer)
 		break;
-
-	    QFontEngine *engine = d->engineForScript( script );
+            if ( ::category( *ch ) != QChar::Mark_NonSpacing && !qIsZeroWidthChar(ch->unicode())) {
+                QFontEngine *engine = d->engineForScript( script );
 #ifdef QT_CHECK_STATE
-	    Q_ASSERT( engine != 0 );
+                Q_ASSERT( engine != 0 );
 #endif // QT_CHECK_STATE
 
-	    glyph_t glyphs[8];
-	    advance_t advances[8];
-	    int nglyphs = 7;
-	    engine->stringToCMap( ch, 1, glyphs, advances, &nglyphs, FALSE );
+                glyph_t glyphs[8];
+                advance_t advances[8];
+                int nglyphs = 7;
+                engine->stringToCMap( ch, 1, glyphs, advances, &nglyphs, FALSE );
 
-	    // ### can nglyphs != 1 happen at all? Not currently I think
-	    if ( uc < QFontEngineData::widthCacheSize && advances[0] < 0x100 )
-		d->engineData->widthCache[ uc ] = advances[0];
-	    width += advances[0];
+                // ### can nglyphs != 1 happen at all? Not currently I think
+                if ( uc < QFontEngineData::widthCacheSize && advances[0] > 0 && advances[0] < 0x100 )
+                    d->engineData->widthCache[ uc ] = advances[0];
+                width += advances[0];
+            }
 	}
 	++pos;
 	++ch;
@@ -2363,17 +2374,22 @@ QRect QFontMetrics::boundingRect( const QString &str, int len ) const
 #endif
 
 /*!
-    Returns the bounding rectangle of the character \a ch relative to
-    the left-most point on the base line.
+    Returns the rectangle that is covered by ink if the character
+    specified by \a ch were to be drawn at the origin of the coordinate
+    system.
 
     Note that the bounding rectangle may extend to the left of (0, 0),
     e.g. for italicized fonts, and that the text output may cover \e
-    all pixels in the bounding rectangle.
+    all pixels in the bounding rectangle. For a space character the rectangle
+    will usually be empty.
 
     Note that the rectangle usually extends both above and below the
     base line.
-
-    \sa width()
+    
+    \warning The width of the returned rectangle is not the advance width
+    of the character. Use boundingRect(const QString &) or width() instead.
+    
+    \sa width() 
 */
 QRect QFontMetrics::boundingRect( QChar ch ) const
 {
@@ -2848,7 +2864,7 @@ bool QFontInfo::fixedPitch() const
 	glyph_t g[2];
 	int l = 2;
 	advance_t a[2];
-	engine->stringToCMap(ch, 2, g, a, &l, false);
+	engine->stringToCMap(ch, 2, g, a, &l, FALSE);
 	engine->fontDef.fixedPitch = a[0] == a[1];
 	engine->fontDef.fixedPitchComputed = TRUE;
     }
@@ -2981,6 +2997,7 @@ void QFontCache::clear()
 	    if ( data->engine )
 		data->engine->deref();
 	    data->engine = 0;
+	    ++it;
 	}
     }
 
@@ -3120,7 +3137,7 @@ void QFontCache::cleanupPrinterFonts()
 		    it.data()->engine->deref();
 		    it.data()->engine = 0;
 		}
-#endif		
+#endif
 		++it;
 	    } else {
 
@@ -3222,7 +3239,7 @@ void QFontCache::timerEvent( QTimerEvent * )
 	}
 
 	// attempt to make up for rounding errors
-	in_use_cost += engineCache.count();
+	in_use_cost += (uint)engineCache.count();
     }
 
     in_use_cost = ( in_use_cost + 512 ) / 1024; // cost is stored in kb
