@@ -1,37 +1,40 @@
 /****************************************************************************
-** $Id: qt/qfont_x11.cpp   3.3.4   edited Oct 18 16:17 $
 **
 ** Implementation of QFont, QFontMetrics and QFontInfo classes for X11
 **
 ** Created : 940515
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
-** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
-** LICENSE.QPL included in the packaging of this file.
+** This file may be used under the terms of the GNU General
+** Public License versions 2.0 or 3.0 as published by the Free
+** Software Foundation and appearing in the files LICENSE.GPL2
+** and LICENSE.GPL3 included in the packaging of this file.
+** Alternatively you may (at your option) use any later version
+** of the GNU General Public License if such license has been
+** publicly approved by Trolltech ASA (or its successors, if any)
+** and the KDE Free Qt Foundation.
 **
-** This file may be distributed and/or modified under the terms of the
-** GNU General Public License version 2 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/.
+** If you are unsure which license is appropriate for your use, please
+** review the following information:
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
 **
-** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
-** licenses for Unix/X11 may use this file in accordance with the Qt Commercial
-** License Agreement provided with the Software.
+** This file may be used under the terms of the Q Public License as
+** defined by Trolltech ASA and appearing in the file LICENSE.QPL
+** included in the packaging of this file.  Licensees holding valid Qt
+** Commercial licenses may use this file in accordance with the Qt
+** Commercial License Agreement provided with the Software.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-**
-** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
-**   information about Qt Commercial License Agreements.
-** See http://www.trolltech.com/qpl/ for QPL licensing information.
-** See http://www.trolltech.com/gpl/ for GPL licensing information.
-**
-** Contact info@trolltech.com if any conditions of this licensing are
-** not clear to you.
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
+** herein.
 **
 **********************************************************************/
 
@@ -68,6 +71,15 @@
 
 Q_EXPORT bool qt_has_xft = FALSE;
 
+#ifndef QT_NO_XFTFREETYPE
+Qt::HANDLE qt_xft_handle(const QFont &font)
+{
+    QFontEngine *engine = font.d->engineForScript( QFontPrivate::defaultScript );
+    if (!engine->type() == QFontEngine::Xft)
+        return 0;
+    return (long)static_cast<QFontEngineXft *>(engine)->font();
+}
+#endif
 
 double qt_pixelSize(double pointSize, QPaintDevice *paintdevice, int scr)
 {
@@ -347,7 +359,7 @@ void QFontPrivate::load( QFont::Script script )
     req.pointSize = 0;
 
     if ( ! engineData ) {
-	QFontCache::Key key( req, QFont::NoScript, screen );
+	QFontCache::Key key( req, QFont::NoScript, screen, paintdevice );
 
 	// look for the requested font in the engine data cache
 	engineData = QFontCache::instance->findEngineData( key );
@@ -658,7 +670,7 @@ int QFontMetrics::width( QChar ch ) const
 	 d->engineData && d->engineData->widthCache[ uc ] )
 	return d->engineData->widthCache[ uc ];
 
-    if ( ::category( ch ) == QChar::Mark_NonSpacing )
+    if ( ::category( ch ) == QChar::Mark_NonSpacing || qIsZeroWidthChar(ch.unicode()))
 	return 0;
 
     QFont::Script script;
@@ -705,7 +717,7 @@ int QFontMetrics::charWidth( const QString &str, int pos ) const
 	QTextEngine layout( cstr.string(), d );
 	layout.itemize( QTextEngine::WidthOnly );
 	width = layout.width( pos-from, 1 );
-    } else if ( ::category( ch ) == QChar::Mark_NonSpacing ) {
+    } else if ( ::category( ch ) == QChar::Mark_NonSpacing || qIsZeroWidthChar(ch.unicode())) {
         width = 0;
     } else {
 	QFontEngine *engine = d->engineForScript( script );
