@@ -1,16 +1,16 @@
 /****************************************************************************
-** $Id: qt/qobject.cpp   3.3.4   edited Jan 5 17:37 $
+** $Id: qt/qobject.cpp   3.3.8   edited Jan 11 14:38 $
 **
 ** Implementation of QObject class
 **
 ** Created : 930418
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
 ** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
+** as defined by Trolltech ASA of Norway and appearing in the file
 ** LICENSE.QPL included in the packaging of this file.
 **
 ** This file may be distributed and/or modified under the terms of the
@@ -834,36 +834,37 @@ void QObject::customEvent( QCustomEvent * )
     class MyMainWindow : public QMainWindow
     {
     public:
-	MyMainWindow( QWidget *parent = 0, const char *name = 0 );
+        MyMainWindow( QWidget *parent = 0, const char *name = 0 );
 
     protected:
-	bool eventFilter( QObject *obj, QEvent *ev );
+        bool eventFilter( QObject *obj, QEvent *ev );
 
     private:
-	QTextEdit *textEdit;
+        QTextEdit *textEdit;
     };
 
     MyMainWindow::MyMainWindow( QWidget *parent, const char *name )
-	: QMainWindow( parent, name )
+        : QMainWindow( parent, name )
     {
-	textEdit = new QTextEdit( this );
-	setCentralWidget( textEdit );
-	textEdit->installEventFilter( this );
+        textEdit = new QTextEdit( this );
+        setCentralWidget( textEdit );
+        textEdit->installEventFilter( this );
     }
 
     bool MyMainWindow::eventFilter( QObject *obj, QEvent *ev )
     {
-	if ( obj == textEdit ) {
-	    if ( e->type() == QEvent::KeyPress ) {
-		qDebug( "Ate key press %d", k->key() );
-		return TRUE;
-	    } else {
-		return FALSE;
-	    }
-	} else {
-	    // pass the event on to the parent class
-	    return QMainWindow::eventFilter( obj, ev );
-	}
+        if ( obj == textEdit ) {
+            if ( e->type() == QEvent::KeyPress ) {
+                QKeyEvent *k = (QKeyEvent*)ev;
+                qDebug( "Ate key press %d", k->key() );
+                return TRUE;
+            } else {
+                return FALSE;
+            }
+        } else {
+            // pass the event on to the parent class
+            return QMainWindow::eventFilter( obj, ev );
+        }
     }
     \endcode
 
