@@ -1,16 +1,16 @@
 /****************************************************************************
-** $Id: qt/qstyle.cpp   3.2.3   edited Jul 8 11:10 $
+** $Id: qt/qstyle.cpp   3.3.8   edited Jan 11 16:03 $
 **
 ** Implementation of QStyle class
 **
 ** Created : 981231
 **
-** Copyright (C) 1998-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1998-2007 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
 ** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
+** as defined by Trolltech ASA of Norway and appearing in the file
 ** LICENSE.QPL included in the packaging of this file.
 **
 ** This file may be distributed and/or modified under the terms of the
@@ -190,6 +190,13 @@ public:
     constructor; otherwise returns FALSE.
 */
 
+/*!
+    \fn int QStyleOption::day() const
+
+    Returns the index of the day in the month if the appropriate
+    constructor was called; otherwise the return value is undefined.
+*/
+
 /*!
     \fn int QStyleOption::lineWidth() const
 
@@ -450,7 +457,6 @@ void QStyle::unPolish( QWidget*)
 
 /*!
     \overload
-
     Late initialization of the QApplication object.
 
     \sa unPolish()
@@ -743,6 +749,7 @@ void QStyle::drawItem( QPainter *p, const QRect &r,
     \value PE_CheckListController controller part of a listview item
     \value PE_CheckListIndicator checkbox part of a listview item
     \value PE_CheckListExclusiveIndicator radiobutton part of a listview item
+    \value PE_RubberBand rubber band used in such things as iconview
 
     \value PE_CustomBase  base value for custom PrimitiveElements.
 	All values above this are reserved for custom use. Custom
@@ -792,7 +799,7 @@ void QStyle::drawItem( QPainter *p, const QRect &r,
 */
 
 /*!
-    \fn void QStyle::drawPrimitive( PrimitiveElement pe, QPainter *p, const QRect &r, const QColorGroup &cg, SFlags flags = Style_Default, const QStyleOption& opt = QStyleOption::Default ) const;
+    \fn void QStyle::drawPrimitive( PrimitiveElement pe, QPainter *p, const QRect &r, const QColorGroup &cg, SFlags flags, const QStyleOption& opt) const
 
     Draws the style PrimitiveElement \a pe using the painter \a p in
     the area \a r. Colors are used from the color group \a cg.
@@ -922,7 +929,7 @@ void QStyle::drawItem( QPainter *p, const QRect &r,
 */
 
 /*!
-    \fn void QStyle::drawControl ( ControlElement element, QPainter *p, const QWidget *widget, const QRect &r, const QColorGroup &cg, SFlags how = Style_Default, const QStyleOption& opt = QStyleOption::Default ) const;
+    \fn void QStyle::drawControl( ControlElement element, QPainter *p, const QWidget *widget, const QRect &r, const QColorGroup &cg, SFlags how, const QStyleOption& opt) const
 
     Draws the ControlElement \a element using the painter \a p in the
     area \a r. Colors are used from the color group \a cg.
@@ -1076,7 +1083,7 @@ void QStyle::drawItem( QPainter *p, const QRect &r,
 */
 
 /*!
-    \fn void QStyle::drawControlMask( ControlElement element, QPainter *p, const QWidget *widget, const QRect &r, const QStyleOption& opt = QStyleOption::Default ) const;
+    \fn void QStyle::drawControlMask( ControlElement element, QPainter *p, const QWidget *widget, const QRect &r, const QStyleOption& opt) const
 
     Draw a bitmask for the ControlElement \a element using the painter
     \a p in the area \a r. See drawControl() for an explanation of the
@@ -1365,7 +1372,7 @@ void QStyle::drawItem( QPainter *p, const QRect &r,
 */
 
 /*!
-    \fn void QStyle::drawComplexControlMask( ComplexControl control, QPainter *p, const QWidget *widget, const QRect &r, const QStyleOption& opt ) const
+    \fn void QStyle::drawComplexControlMask( ComplexControl control, QPainter *p, const QWidget *widget, const QRect &r, const QStyleOption& opt) const
 
     Draw a bitmask for the ComplexControl \a control using the painter
     \a p in the area \a r. See drawComplexControl() for an explanation
@@ -1465,6 +1472,9 @@ void QStyle::drawItem( QPainter *p, const QRect &r,
 
     \value PM_MenuBarFrameWidth  frame width of a menubar.
 
+    \value PM_MenuBarItemSpacing  spacing between menubar items.
+    \value PM_ToolBarItemSpacing  spacing between toolbar items.
+
     \value PM_TabBarTabOverlap number of pixels the tabs should overlap.
     \value PM_TabBarTabHSpace extra space added to the tab width.
     \value PM_TabBarTabVSpace extra space added to the tab height.
@@ -1472,6 +1482,7 @@ void QStyle::drawItem( QPainter *p, const QRect &r,
 	and the tab pages.
     \value PM_TabBarBaseOverlap number of pixels the tab bar overlaps
 	the tab bar base.
+    \value PM_TabBarScrollButtonWidth
     \value PM_TabBarTabShiftHorizontal horizontal pixel shift when a
         tab is selected.
     \value PM_TabBarTabShiftVertical vertical pixel shift when a
@@ -1739,6 +1750,9 @@ void QStyle::drawItem( QPainter *p, const QRect &r,
 	values above this are reserved for custom use. Therefore,
 	custom values must be greater than this value.
 
+    \value SH_ToolButton_Uses3D indicates whether QToolButtons should
+    use a 3D frame when the mouse is over them
+
     \value SH_ToolBox_SelectedPageTitleBold Boldness of the selected
     page title in a QToolBox.
 
@@ -1747,6 +1761,8 @@ void QStyle::drawItem( QPainter *p, const QRect &r,
 
     \value SH_Table_GridLineColor
 
+    \value SH_UnderlineAccelerator whether accelerators are underlined
+
     \sa styleHint()
 */
 
