@@ -1,16 +1,16 @@
 /****************************************************************************
-** $Id: qt/qwidget_x11.cpp   3.2.3   edited Jul 23 00:07 $
+** $Id: qt/qwidget_x11.cpp   3.3.8   edited Jan 11 14:38 $
 **
 ** Implementation of QWidget and QWindow classes for X11
 **
 ** Created : 931031
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
 ** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
+** as defined by Trolltech ASA of Norway and appearing in the file
 ** LICENSE.QPL included in the packaging of this file.
 **
 ** This file may be distributed and/or modified under the terms of the
@@ -78,13 +78,14 @@ extern bool qt_nograb();
 // defined in qapplication_x11.cpp
 extern void qt_deferred_map_add( QWidget* );
 extern void qt_deferred_map_take( QWidget* );
+extern bool qt_deferred_map_contains(QWidget *);
 
 static QWidget *mouseGrb    = 0;
 static QWidget *keyboardGrb = 0;
 
 // defined in qapplication_x11.cpp
 extern Time qt_x_time;
-extern bool qt_use_xrender;
+extern Time qt_x_user_time;
 
 // defined in qfont_x11.cpp
 extern bool qt_has_xft;
@@ -96,6 +97,8 @@ int qt_x11_create_desktop_on_screen = -1;
  *****************************************************************************/
 
 // defined in qapplication_x11.cpp
+extern Atom qt_wm_state;
+extern Atom qt_wm_change_state;
 extern Atom qt_wm_delete_window;
 extern Atom qt_wm_take_focus;
 extern Atom qt_wm_client_leader;
@@ -106,18 +109,24 @@ extern Atom qt_net_wm_context_help;
 extern Atom qt_net_wm_ping;
 extern Atom qt_xa_motif_wm_hints;
 extern Atom qt_net_wm_name;
+extern Atom qt_net_wm_icon_name;
 extern Atom qt_net_wm_state;
 extern Atom qt_net_wm_state_modal;
 extern Atom qt_net_wm_state_max_v;
 extern Atom qt_net_wm_state_max_h;
+extern Atom qt_net_wm_state_fullscreen;
+extern Atom qt_net_wm_state_above;
 extern Atom qt_net_wm_state_stays_on_top;
 extern Atom qt_net_wm_window_type;
 extern Atom qt_net_wm_window_type_normal;
 extern Atom qt_net_wm_window_type_dialog;
 extern Atom qt_net_wm_window_type_toolbar;
+extern Atom qt_net_wm_window_type_menu;
+extern Atom qt_net_wm_window_type_utility;
 extern Atom qt_net_wm_window_type_splash;
 extern Atom qt_net_wm_window_type_override;
 extern Atom qt_net_wm_pid;
+extern Atom qt_net_wm_user_time;
 extern Atom qt_enlightenment_desktop;
 extern Atom qt_net_virtual_roots;
 extern bool qt_broken_wm;
@@ -145,7 +154,7 @@ const uint stdWidgetEventMask =			// X event mask
 	    FocusChangeMask |
 	    ExposureMask |
 	    PropertyChangeMask |
-	    StructureNotifyMask | SubstructureRedirectMask
+	    StructureNotifyMask
 	);
 
 const uint stdDesktopEventMask =			// X event mask
@@ -191,6 +200,79 @@ Q_EXPORT void qt_x11_enforce_cursor( QWidget * w )
     }
 }
 
+Q_EXPORT void qt_wait_for_window_manager( QWidget* w )
+{
+    QApplication::flushX();
+    XEvent ev;
+    QTime t;
+    t.start();
+    while ( !XCheckTypedWindowEvent( w->x11Display(), w->winId(), ReparentNotify, &ev ) ) {
+	if ( XCheckTypedWindowEvent( w->x11Display(), w->winId(), MapNotify, &ev ) )
+	    break;
+	if ( t.elapsed() > 500 )
+	    return; // give up, no event available
+	qApp->syncX(); // non-busy wait
+    }
+    qApp->x11ProcessEvent( &ev );
+    if ( XCheckTypedWindowEvent( w->x11Display(), w->winId(), ConfigureNotify, &ev ) )
+	qApp->x11ProcessEvent( &ev );
+}
+
+static void qt_net_change_wm_state(const QWidget* w, bool set, Atom one, Atom two = 0)
+{
+    if (w->isShown()) {
+	// managed by WM
+        XEvent e;
+        e.xclient.type = ClientMessage;
+        e.xclient.message_type = qt_net_wm_state;
+        e.xclient.display = w->x11Display();
+        e.xclient.window = w->winId();
+        e.xclient.format = 32;
+        e.xclient.data.l[ 0 ] = set ? 1 : 0;
+        e.xclient.data.l[ 1 ] = one;
+        e.xclient.data.l[ 2 ] = two;
+        e.xclient.data.l[ 3 ] = 0;
+        e.xclient.data.l[ 4 ] = 0;
+        XSendEvent(w->x11Display(), RootWindow(w->x11Display(), w->x11Screen()),
+		   False, (SubstructureNotifyMask|SubstructureRedirectMask), &e);
+    } else {
+        Atom ret;
+	int format = 0, status;
+	unsigned char *data = 0;
+	unsigned long nitems = 0, after = 0;
+	Atom *old_states = 0;
+	status = XGetWindowProperty(w->x11Display(), w->winId(),
+				    qt_net_wm_state, 0, 1024, False,
+				    XA_ATOM, &ret, &format, &nitems,
+				    &after, &data);
+	if (status == Success && ret == XA_ATOM && format == 32 && nitems > 0)
+	    old_states = (Atom *) data;
+	else
+	    nitems = 0;
+
+	Atom *new_states = new Atom[nitems + 2];
+	int i, j = 0;
+	for (i = 0; i < (int)nitems; ++i) {
+	    if (old_states[i] && old_states[i] != one && old_states[i] != two)
+		new_states[j++] = old_states[i];
+	}
+
+	if (set) {
+	    if (one) new_states[j++] = one;
+	    if (two) new_states[j++] = two;
+	}
+
+	if (j)
+	    XChangeProperty(w->x11Display(), w->winId(), qt_net_wm_state, XA_ATOM, 32,
+			    PropModeReplace, (uchar *) new_states, j);
+	else
+	    XDeleteProperty(w->x11Display(), w->winId(), qt_net_wm_state);
+
+	delete [] new_states;
+	if (data) XFree(data);
+    }
+}
+
 /*!
     Creates a new widget window if \a window is 0, otherwise sets the
     widget's window to \a window.
@@ -361,17 +443,17 @@ void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
 	rendhd = 0;
     }
 
-    if ( qt_use_xrender && qt_has_xft )
+    if ( qt_has_xft )
 	rendhd = (HANDLE) XftDrawCreate( dpy, id, (Visual *) x11Visual(),
 					 x11Colormap() );
 #endif // QT_NO_XFTFREETYPE
 
     // NET window types
-    long net_wintypes[4] = { 0, 0, 0, 0 };
+    long net_wintypes[7] = { 0, 0, 0, 0, 0, 0, 0 };
     int curr_wintype = 0;
 
     // NET window states
-    long net_winstates[4] = { 0, 0, 0, 0 };
+    long net_winstates[6] = { 0, 0, 0, 0, 0, 0 };
     int curr_winstate = 0;
 
     struct {
@@ -389,7 +471,7 @@ void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
 	ulong wsa_mask = 0;
 
 	if ( testWFlags(WStyle_Splash) ) {
-            if ( qt_net_supports(qt_net_wm_window_type_splash) ) {
+            if (qt_net_supports(qt_net_wm_window_type_splash)) {
                 clearWFlags( WX11BypassWM );
                 net_wintypes[curr_wintype++] = qt_net_wm_window_type_splash;
 	    } else {
@@ -425,16 +507,10 @@ void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
 	    if (testWFlags(WStyle_Tool)) {
 		wsa.save_under = True;
 		wsa_mask |= CWSaveUnder;
-
-		// toolbar netwm type
-		net_wintypes[curr_wintype++] = qt_net_wm_window_type_toolbar;
 	    }
 	} else if (testWFlags(WType_Dialog)) {
 	    setWFlags(WStyle_NormalBorder | WStyle_Title |
 		      WStyle_SysMenu | WStyle_ContextHelp);
-
-	    // dialog netwm type
-            net_wintypes[curr_wintype++] = qt_net_wm_window_type_dialog;
 	} else {
 	    setWFlags(WStyle_NormalBorder | WStyle_Title |
 		      WStyle_MinMax | WStyle_SysMenu);
@@ -446,12 +522,28 @@ void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
 	    }
 	}
 
+	// ### need a better way to do this
+	if (inherits("QPopupMenu")) {
+	    // menu netwm type
+	    net_wintypes[curr_wintype++] = qt_net_wm_window_type_menu;
+	} else if (inherits("QToolBar")) {
+	    // toolbar netwm type
+	    net_wintypes[curr_wintype++] = qt_net_wm_window_type_toolbar;
+	} else if (testWFlags(WStyle_Customize) && testWFlags(WStyle_Tool)) {
+	    // utility netwm type
+	    net_wintypes[curr_wintype++] = qt_net_wm_window_type_utility;
+	}
+
+	if (dialog) // dialog netwm type
+            net_wintypes[curr_wintype++] = qt_net_wm_window_type_dialog;
 	// normal netwm type - default
 	net_wintypes[curr_wintype++] = qt_net_wm_window_type_normal;
 
 	// stays on top
-	if (testWFlags(WStyle_StaysOnTop))
+	if (testWFlags(WStyle_StaysOnTop)) {
+	    net_winstates[curr_winstate++] = qt_net_wm_state_above;
 	    net_winstates[curr_winstate++] = qt_net_wm_state_stays_on_top;
+	}
 
         if (testWFlags(WShowModal)) {
             mwmhints.input_mode = 3L; // MWM_INPUT_FULL_APPLICATION_MODAL
@@ -486,13 +578,8 @@ void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
 	if (p)
 	    p = p->topLevelWidget();
 
-	if ( testWFlags(WStyle_DialogBorder) ||
-	     testWFlags(WStyle_StaysOnTop) ||
-	     dialog ||
-	     testWFlags(WStyle_Tool) ) {
-	    if ( testWFlags( WStyle_StaysOnTop ) )
-		XSetTransientForHint( dpy, id, None );
-	    else if ( p )
+	if (dialog || testWFlags(WStyle_DialogBorder) || testWFlags(WStyle_Tool)) {
+	    if ( p )
 		XSetTransientForHint( dpy, id, p->winId() );
 	    else				// application-modal
 		XSetTransientForHint( dpy, id, root_win );
@@ -508,27 +595,27 @@ void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
 	size_hints.y = crect.top();
 	size_hints.width = crect.width();
 	size_hints.height = crect.height();
-	size_hints.win_gravity = QApplication::reverseLayout() ?
-				 NorthEastGravity : NorthWestGravity;
-	const char *title = qAppName();
+	size_hints.win_gravity =
+	    QApplication::reverseLayout() ? NorthEastGravity : NorthWestGravity;
+
 	XWMHints wm_hints;			// window manager hints
 	wm_hints.input = True;
 	wm_hints.initial_state = NormalState;
 	wm_hints.flags = InputHint | StateHint;
+	if ( !qt_x11_wm_client_leader )
+	    qt_x11_create_wm_client_leader();
 
-	if ( p && ! p->isDesktop() ) {
-            // the real client leader (head of the group)
-	    wm_hints.window_group = p->winId();
-	    wm_hints.flags |= WindowGroupHint;
-	}
+	wm_hints.window_group = qt_x11_wm_client_leader;
+	wm_hints.flags |= WindowGroupHint;
 
 	XClassHint class_hint;
-	class_hint.res_class = (char*) title;	// app name
-	class_hint.res_name = (char*) title; 	// maybe some day we need a function for that
-	XSetWMProperties( dpy, id, 0, 0, 0, 0, &size_hints, &wm_hints,
-			  &class_hint );
+	class_hint.res_name = (char *) qAppName(); // application name
+	class_hint.res_class = (char *) qAppClass();	// application class
+
+       	XSetWMProperties( dpy, id, 0, 0, 0, 0, &size_hints, &wm_hints, &class_hint );
+
 	XResizeWindow( dpy, id, crect.width(), crect.height() );
-	XStoreName( dpy, id, (char*) title );
+	XStoreName( dpy, id, qAppName() );
 	Atom protocols[4];
 	int n = 0;
 	protocols[n++] = qt_wm_delete_window;	// support del window protocol
@@ -542,16 +629,22 @@ void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
         if ( mwmhints.flags != 0l )
             XChangeProperty(dpy, id, qt_xa_motif_wm_hints, qt_xa_motif_wm_hints, 32,
                             PropModeReplace, (unsigned char *) &mwmhints, 5);
+        else
+            XDeleteProperty(dpy, id, qt_xa_motif_wm_hints);
 
 	// set _NET_WM_WINDOW_TYPE
 	if (curr_wintype > 0)
 	    XChangeProperty(dpy, id, qt_net_wm_window_type, XA_ATOM, 32, PropModeReplace,
 			    (unsigned char *) net_wintypes, curr_wintype);
+        else
+            XDeleteProperty(dpy, id, qt_net_wm_window_type);
 
 	// set _NET_WM_WINDOW_STATE
 	if (curr_winstate > 0)
 	    XChangeProperty(dpy, id, qt_net_wm_state, XA_ATOM, 32, PropModeReplace,
 			    (unsigned char *) net_winstates, curr_winstate);
+        else
+            XDeleteProperty(dpy, id, qt_net_wm_state);
 
 	// set _NET_WM_PID
 	long curr_pid = getpid();
@@ -566,9 +659,6 @@ void QWidget::create( WId window, bool initializeWindow, bool destroyOldWindow)
 			 qt_window_role, XA_STRING, 8, PropModeReplace,
 			 (unsigned char *)name(), qstrlen( name() ) );
 
-	if ( !qt_x11_wm_client_leader )
-	    qt_x11_create_wm_client_leader();
-
 	// set client leader property
 	XChangeProperty( dpy, id, qt_wm_client_leader,
 			 XA_WINDOW, 32, PropModeReplace,
@@ -728,7 +818,8 @@ void QWidget::reparentSys( QWidget *parent, WFlags f, const QPoint &p, bool show
 
     const QObjectList *chlist = children();
     if ( chlist ) {				// reparent children
-	QObjectListIt it( *chlist );
+        QObjectList childList(*chlist);
+	QObjectListIt it(childList); // iterate over copy
 	QObject *obj;
 	while ( (obj=it.current()) ) {
 	    if ( obj->isWidgetType() ) {
@@ -762,7 +853,17 @@ void QWidget::reparentSys( QWidget *parent, WFlags f, const QPoint &p, bool show
     else
 	XSetWindowBackground( dpy, winid, bgc.pixel(x11Screen()) );
 
-    setGeometry( p.x(), p.y(), s.width(), s.height() );
+    if (isTopLevel()) {
+        // preserve maximized/fullscreen flags and the normal geometry
+        uint save_state = widget_state & (WState_Maximized | WState_FullScreen);
+        const QRect r = topData()->normalGeometry;
+        setGeometry(p.x(), p.y(), s.width(), s.height());
+        widget_state |= save_state;
+        topData()->normalGeometry = r;
+    } else {
+        setGeometry(p.x(), p.y(), s.width(), s.height());
+    }
+
     setEnabled( enable );
     setFocusPolicy( fp );
     if ( !capt.isNull() ) {
@@ -1085,10 +1186,15 @@ void QWidget::setIcon( const QPixmap &pixmap )
 
 void QWidget::setIconText( const QString &iconText )
 {
-    createTLExtra();
-    extra->topextra->iconText = iconText;
-    XSetIconName( x11Display(), winId(), iconText.utf8() );
+    if (QWidget::iconText() == iconText)
+        return;
+
+    topData()->iconText = iconText;
     XSetWMIconName( x11Display(), winId(), qstring_to_xtp(iconText) );
+
+    QCString net_wm_icon_name = iconText.utf8();
+    XChangeProperty(x11Display(), winId(), qt_net_wm_icon_name, qt_utf8_string, 8, PropModeReplace,
+		    (unsigned char *) net_wm_icon_name.data(), net_wm_icon_name.length());
 }
 
 void QWidget::setMouseTracking( bool enable )
@@ -1334,18 +1440,9 @@ QWidget *QWidget::keyboardGrabber()
 void QWidget::setActiveWindow()
 {
     QWidget *tlw = topLevelWidget();
-    if ( tlw->isVisible() && !tlw->topData()->embedded ) {
+    if ( tlw->isVisible() && !tlw->topData()->embedded && !qt_deferred_map_contains(tlw) ) {
 	XSetInputFocus( x11Display(), tlw->winId(), RevertToNone, qt_x_time);
-
-#ifndef QT_NO_XIM
-	// trigger input context creation if it hasn't happened already
-	createInputContext();
-
-	if (tlw->topData()->xic) {
-	    QInputContext *qic = (QInputContext *) tlw->topData()->xic;
-	    qic->setFocus();
-	}
-#endif
+        focusInputContext();
     }
 }
 
@@ -1532,6 +1629,126 @@ void QWidget::repaint( const QRegion& reg, bool erase )
     Erases the widget region \a r if \a erase is TRUE.
 */
 
+void QWidget::setWindowState(uint newstate)
+{
+    bool needShow = FALSE;
+    uint oldstate = windowState();
+    if (isTopLevel()) {
+        QTLWExtra *top = topData();
+
+	if ((oldstate & WindowMaximized) != (newstate & WindowMaximized)) {
+	    if (qt_net_supports(qt_net_wm_state_max_h) && qt_net_supports(qt_net_wm_state_max_v)) {
+		qt_net_change_wm_state(this, (newstate & WindowMaximized),
+				       qt_net_wm_state_max_h, qt_net_wm_state_max_v);
+	    } else if (! (newstate & WindowFullScreen)) {
+		if (newstate & WindowMaximized) {
+		    // save original geometry
+                    const QRect normalGeometry = geometry();
+
+		    if (isVisible()) {
+			updateFrameStrut();
+			const QRect maxRect = QApplication::desktop()->availableGeometry(this);
+			const QRect r = top->normalGeometry;
+			setGeometry(maxRect.x() + top->fleft,
+				    maxRect.y() + top->ftop,
+				    maxRect.width() - top->fleft - top->fright,
+				    maxRect.height() - top->ftop - top->fbottom);
+                        top->normalGeometry = r;
+		    }
+
+                    if (top->normalGeometry.width() < 0)
+			top->normalGeometry = normalGeometry;
+		} else {
+		    // restore original geometry
+		    setGeometry(top->normalGeometry);
+		}
+	    }
+	}
+
+	if ((oldstate & WindowFullScreen) != (newstate & WindowFullScreen)) {
+	    if (qt_net_supports(qt_net_wm_state_fullscreen)) {
+		qt_net_change_wm_state(this, (newstate & WindowFullScreen),
+				       qt_net_wm_state_fullscreen);
+	    } else {
+                needShow = isVisible();
+
+		if (newstate & WindowFullScreen) {
+                    const QRect normalGeometry = QRect(pos(), size());
+
+		    top->savedFlags = getWFlags();
+                    reparent(0, WType_TopLevel | WStyle_Customize | WStyle_NoBorder |
+			     // preserve some widget flags
+			     (getWFlags() & 0xffff0000),
+			     mapToGlobal(QPoint(0, 0)));
+                    const QRect r = top->normalGeometry;
+                    setGeometry(qApp->desktop()->screenGeometry(this));
+                    top->normalGeometry = r;
+
+                    if ( top->normalGeometry.width() < 0 )
+			top->normalGeometry = normalGeometry;
+                } else {
+		    reparent( 0, top->savedFlags, mapToGlobal(QPoint(0, 0)) );
+
+                    if (newstate & WindowMaximized) {
+                        // from fullscreen to maximized
+			updateFrameStrut();
+			const QRect maxRect = QApplication::desktop()->availableGeometry(this);
+			const QRect r = top->normalGeometry;
+			setGeometry(maxRect.x() + top->fleft,
+				    maxRect.y() + top->ftop,
+				    maxRect.width() - top->fleft - top->fright,
+				    maxRect.height() - top->ftop - top->fbottom);
+                        top->normalGeometry = r;
+                    } else {
+                        // restore original geometry
+                        setGeometry(top->normalGeometry);
+                    }
+		}
+	    }
+	}
+
+	if ((oldstate & WindowMinimized) != (newstate & WindowMinimized)) {
+	    if (isVisible()) {
+		if (newstate & WindowMinimized) {
+		    XEvent e;
+		    e.xclient.type = ClientMessage;
+		    e.xclient.message_type = qt_wm_change_state;
+		    e.xclient.display = x11Display();
+		    e.xclient.window = winid;
+		    e.xclient.format = 32;
+		    e.xclient.data.l[0] = IconicState;
+		    e.xclient.data.l[1] = 0;
+		    e.xclient.data.l[2] = 0;
+		    e.xclient.data.l[3] = 0;
+		    e.xclient.data.l[4] = 0;
+		    XSendEvent(x11Display(), RootWindow(x11Display(), x11Screen()),
+			       False, (SubstructureNotifyMask|SubstructureRedirectMask), &e);
+		} else {
+		    XMapWindow(x11Display(), winId());
+		}
+	    }
+
+	    needShow = FALSE;
+	}
+    }
+
+    widget_state &= ~(WState_Minimized | WState_Maximized | WState_FullScreen);
+    if (newstate & WindowMinimized)
+	widget_state |= WState_Minimized;
+    if (newstate & WindowMaximized)
+	widget_state |= WState_Maximized;
+    if (newstate & WindowFullScreen)
+	widget_state |= WState_FullScreen;
+
+    if (needShow)
+	show();
+
+    if (newstate & WindowActive)
+	setActiveWindow();
+
+    QEvent e(QEvent::WindowStateChange);
+    QApplication::sendEvent(this, &e);
+}
 
 /*!
   \internal
@@ -1540,30 +1757,64 @@ void QWidget::repaint( const QRegion& reg, bool erase )
 
 void QWidget::showWindow()
 {
-
     if ( isTopLevel()  ) {
-	int sm = topData()->showMode;
-	if ( sm ) { // handles minimize and reset
-	    XWMHints *h = XGetWMHints( x11Display(), winId() );
-	    XWMHints  wm_hints;
-	    bool got_hints = h != 0;
-	    if ( !got_hints ) {
-		h = &wm_hints;
-		h->flags = 0;
-	    }
-	    h->initial_state = sm == 1? IconicState : NormalState;
-	    h->flags |= StateHint;
-	    XSetWMHints( x11Display(), winId(), h );
-	    if ( got_hints )
-		XFree( (char *)h );
-	    topData()->showMode = sm == 1?3:0; // trigger reset to normal state next time
+	XWMHints *h = XGetWMHints( x11Display(), winId() );
+	XWMHints  wm_hints;
+	bool got_hints = h != 0;
+	if ( !got_hints ) {
+	    h = &wm_hints;
+	    h->flags = 0;
 	}
-	if ( topData()->parentWinId &&
-	     topData()->parentWinId != QPaintDevice::x11AppRootWindow(x11Screen()) &&
-	     !isMinimized() ) {
+	h->initial_state = testWState(WState_Minimized) ? IconicState : NormalState;
+	h->flags |= StateHint;
+	XSetWMHints( x11Display(), winId(), h );
+	if ( got_hints )
+	    XFree( (char *)h );
+
+	if (qt_x_user_time != CurrentTime) {
+	    XChangeProperty(x11Display(), winId(), qt_net_wm_user_time, XA_CARDINAL,
+			    32, PropModeReplace, (unsigned char *) &qt_x_user_time, 1);
+	}
+
+	if (!topData()->embedded &&
+	    topData()->parentWinId &&
+	    topData()->parentWinId != QPaintDevice::x11AppRootWindow(x11Screen()) &&
+	    !isMinimized() ) {
 	    qt_deferred_map_add( this );
 	    return;
 	}
+
+ 	if (isMaximized() && !isFullScreen()
+	    && !(qt_net_supports(qt_net_wm_state_max_h)
+		 && qt_net_supports(qt_net_wm_state_max_v))) {
+ 	    XMapWindow( x11Display(), winId() );
+ 	    qt_wait_for_window_manager(this);
+
+ 	    // if the wm was not smart enough to adjust our size, do that manually
+ 	    updateFrameStrut();
+	    QRect maxRect = QApplication::desktop()->availableGeometry(this);
+
+ 	    QTLWExtra *top = topData();
+ 	    QRect normalRect = top->normalGeometry;
+
+ 	    setGeometry(maxRect.x() + top->fleft,
+			maxRect.y() + top->ftop,
+ 			maxRect.width() - top->fleft - top->fright,
+ 			maxRect.height() - top->ftop - top->fbottom);
+
+	    // restore the original normalGeometry
+ 	    top->normalGeometry = normalRect;
+ 	    // internalSetGeometry() clears the maximized flag... make sure we set it back
+ 	    setWState(WState_Maximized);
+
+ 	    return;
+ 	}
+
+	if (isFullScreen() && !qt_net_supports(qt_net_wm_state_fullscreen)) {
+ 	    XMapWindow(x11Display(), winId());
+ 	    qt_wait_for_window_manager(this);
+	    return;
+	}
     }
     XMapWindow( x11Display(), winId() );
 }
@@ -1597,163 +1848,6 @@ void QWidget::hideWindow()
     }
 }
 
-
-/*!
-    Shows the widget minimized, as an icon.
-
-    Calling this function only affects \link isTopLevel() top-level
-    widgets\endlink.
-
-    \sa showNormal(), showMaximized(), show(), hide(), isVisible(),
-    isMinimized()
-*/
-
-void QWidget::showMinimized()
-{
-    if ( isTopLevel() ) {
-	if ( isVisible() && !isMinimized() )
-	    XIconifyWindow( x11Display(), winId(), x11Screen() );
-	else {
-	    topData()->showMode = 1;
-	    show();
-	}
-    } else {
-	show();
-    }
-    QEvent e( QEvent::ShowMinimized );
-    QApplication::sendEvent( this, &e );
-    clearWState( WState_Maximized );
-    setWState( WState_Minimized );
-}
-
-bool QWidget::isMinimized() const
-{
-    // true for non-toplevels that have the minimized flag, e.g. MDI children
-    return (isTopLevel() && qt_wstate_iconified( winId() )) || ( !isTopLevel() && testWState( WState_Minimized ) );
-}
-
-bool QWidget::isMaximized() const
-{
-    return testWState(WState_Maximized);
-}
-
-Q_EXPORT void qt_wait_for_window_manager( QWidget* w )
-{
-    QApplication::flushX();
-    XEvent ev;
-    QTime t;
-    t.start();
-    while ( !XCheckTypedWindowEvent( w->x11Display(), w->winId(), ReparentNotify, &ev ) ) {
-	if ( XCheckTypedWindowEvent( w->x11Display(), w->winId(), MapNotify, &ev ) )
-	    break;
-	if ( t.elapsed() > 500 )
-	    return; // give up, no event available
-	qApp->syncX(); // non-busy wait
-    }
-    qApp->x11ProcessEvent( &ev );
-    if ( XCheckTypedWindowEvent( w->x11Display(), w->winId(), ConfigureNotify, &ev ) )
-	qApp->x11ProcessEvent( &ev );
-}
-
-/*!
-    Shows the widget maximized.
-
-    Calling this function only affects \link isTopLevel() top-level
-    widgets\endlink.
-
-    On X11, this function may not work properly with certain window
-    managers. See the \link geometry.html Window Geometry
-    documentation\endlink for an explanation.
-
-    \sa showNormal(), showMinimized(), show(), hide(), isVisible()
-*/
-
-void QWidget::showMaximized()
-{
-    if ( isTopLevel() ) {
-	Display *dpy = x11Display();
-
-	if ( qt_net_supports(qt_net_wm_state_max_h) &&
-	    qt_net_supports(qt_net_wm_state_max_v)) {
-	    // we have a NET supporting window manager
-	    long net_winstates[3] = { 0, 0, 0 };
-	    int curr_winstate = 0;
-
-	    // keep modal state if it is set
-	    if (testWFlags(WShowModal))
-		net_winstates[curr_winstate++] = qt_net_wm_state_modal;
-
-	    // set maximized states
-	    net_winstates[curr_winstate++] = qt_net_wm_state_max_v;
-	    net_winstates[curr_winstate++] = qt_net_wm_state_max_h;
-
-	    XChangeProperty(dpy, winId(), qt_net_wm_state, XA_ATOM, 32, PropModeReplace,
-			    (unsigned char *) net_winstates, curr_winstate);
-	} else {
-	    int scr = x11Screen();
-	    int sw = DisplayWidth(dpy,scr);
-	    int sh = DisplayHeight(dpy,scr);
-	    if ( topData()->normalGeometry.width() < 0 )
-		topData()->normalGeometry = geometry();
-	    if ( !topData()->parentWinId && !isVisible() ) {
-		setGeometry(0, 0, sw, sh );
-		show();
-		qt_wait_for_window_manager( this );
-		updateFrameStrut();
-	    }
-	    if ( width() == sw && height() == sh ) {
-		// the wm was not smart enough to adjust our size, do that manually
-		sw -= frameGeometry().width() - width();
-		sh -= frameGeometry().height() - height();
-		resize( sw, sh );
-	    }
-	}
-    }
-    show();
-    QEvent e( QEvent::ShowMaximized );
-    QApplication::sendEvent( this, &e );
-    clearWState( WState_Minimized );
-    setWState(WState_Maximized);
-}
-
-/*!
-    Restores the widget after it has been maximized or minimized.
-
-    Calling this function only affects \link isTopLevel() top-level
-    widgets\endlink.
-
-    \sa showMinimized(), showMaximized(), show(), hide(), isVisible()
-*/
-
-void QWidget::showNormal()
-{
-    if ( isTopLevel() ) {
-	if ( topData()->fullscreen ) {
-	    // when reparenting, preserve some widget flags
-	    reparent( 0, topData()->savedFlags, QPoint(0,0) );
-	    topData()->fullscreen = 0;
-	}
-	QRect r = topData()->normalGeometry;
-	if ( r.width() >= 0 ) {
-	    // the widget has been maximized
-	    topData()->normalGeometry = QRect(0,0,-1,-1);
-	    resize( r.size() );
-	    move( r.topLeft() );
-	}
-    }
-    if ( extra && extra->topextra )
-	extra->topextra->fullscreen = 0;
-    if ( !isVisible() ) {
-	show();
-    } else {
-	showWindow();
-    }
-    QEvent e( QEvent::ShowNormal );
-    QApplication::sendEvent( this, &e );
-    clearWState( WState_Minimized | WState_Maximized );
-}
-
-
 /*!
     Raises this widget to the top of the parent widget's stack.
 
@@ -1820,7 +1914,6 @@ void QWidget::stackUnder( QWidget* w)
   a negative position.
 */
 
-// ### this should be set to NorthEast for reversed layout!
 int qt_widget_tlw_gravity = NorthWestGravity;
 
 static void do_size_hints( QWidget* widget, QWExtra *x )
@@ -1863,8 +1956,9 @@ static void do_size_hints( QWidget* widget, QWExtra *x )
     }
     s.flags |= PWinGravity;
     s.win_gravity = qt_widget_tlw_gravity;	// usually NorthWest
-    // ### should this be set to NE for right to left languages.
-    qt_widget_tlw_gravity = NorthWestGravity;	// reset in case it was set
+    // reset in case it was set
+    qt_widget_tlw_gravity =
+        QApplication::reverseLayout() ? NorthEastGravity : NorthWestGravity;
     XSetWMNormalHints( widget->x11Display(), widget->winId(), &s );
 }
 
@@ -1875,7 +1969,18 @@ void QWidget::internalSetGeometry( int x, int y, int w, int h, bool isMove )
 
     if ( testWFlags(WType_Desktop) )
 	return;
-    clearWState(WState_Maximized);
+    if (isTopLevel()) {
+        if (!qt_net_supports(qt_net_wm_state_max_h)
+            && !qt_net_supports(qt_net_wm_state_max_v))
+            clearWState(WState_Maximized);
+        if (!qt_net_supports(qt_net_wm_state_fullscreen))
+            clearWState(WState_FullScreen);
+	topData()->normalGeometry = QRect(0, 0, -1, -1);
+    } else {
+        // for QWorkspace
+        clearWState(WState_Maximized);
+        clearWState(WState_FullScreen);
+    }
     if ( extra ) {				// any size restrictions?
 	w = QMIN(w,extra->maxw);
 	h = QMIN(h,extra->maxh);
@@ -2120,7 +2225,9 @@ void QWidget::erase( const QRegion& reg )
     that is read but not written. For example, when scrolling 10
     pixels rightwards, the leftmost ten pixels of the widget need
     repainting. The paint event may be delivered immediately or later,
-    depending on some heuristics.
+    depending on some heuristics (note that you might have to force
+    processing of paint events using QApplication::sendPostedEvents()
+    when using scroll() and move() in combination).
 
     \sa QScrollView erase() bitBlt()
 */
@@ -2417,13 +2524,19 @@ void QWidget::setMask( const QRegion& region )
 
 /*!
     Causes only the pixels of the widget for which \a bitmap has a
-    corresponding 1 bit to be visible. If the region includes pixels
-    outside the rect() of the widget, window system controls in that
-    area may or may not be visible, depending on the platform.
+    corresponding 1 bit to be visible. Use Qt::color0 to draw
+    transparent regions and Qt::color1 to draw opaque regions of the
+    bitmap.
+
+    If the region includes pixels outside the rect() of the widget,
+    window system controls in that area may or may not be visible,
+    depending on the platform.
 
     Note that this effect can be slow if the region is particularly
     complex.
 
+    See \c examples/tux for an example of masking for transparency.
+
     \sa setMask(), clearMask()
 */
 
@@ -2584,7 +2697,7 @@ void QWidget::destroyInputContext()
 void QWidget::resetInputContext()
 {
 #ifndef QT_NO_XIM
-    if (qt_xim_style & XIMPreeditCallbacks) {
+    if ((qt_xim_style & XIMPreeditCallbacks) && hasFocus()) {
 	QWidget *tlw = topLevelWidget();
 	QTLWExtra *topdata = tlw->topData();
 
@@ -2604,14 +2717,25 @@ void QWidget::focusInputContext()
 {
 #ifndef QT_NO_XIM
     QWidget *tlw = topLevelWidget();
-    QTLWExtra *topdata = tlw->topData();
+    if (!tlw->isPopup() || isInputMethodEnabled()) {
+        QTLWExtra *topdata = tlw->topData();
 
-    // trigger input context creation if it hasn't happened already
-    createInputContext();
+        // trigger input context creation if it hasn't happened already
+        createInputContext();
 
-    if (topdata->xic) {
-	QInputContext *qic = (QInputContext *) topdata->xic;
-	qic->setFocus();
+        if (topdata->xic) {
+            QInputContext *qic = (QInputContext *) topdata->xic;
+            qic->setFocus();
+        }
     }
 #endif // QT_NO_XIM
 }
+
+void QWidget::setWindowOpacity(double)
+{
+}
+
+double QWidget::windowOpacity() const
+{
+    return 1.0;
+}
