@@ -1,37 +1,40 @@
 /****************************************************************************
-** $Id: qt/qgl.cpp   3.3.4   edited Aug 10 18:40 $
 **
 ** Implementation of OpenGL classes for Qt
 **
 ** Created : 970112
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the opengl module of the Qt GUI Toolkit.
 **
-** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
-** LICENSE.QPL included in the packaging of this file.
+** This file may be used under the terms of the GNU General
+** Public License versions 2.0 or 3.0 as published by the Free
+** Software Foundation and appearing in the files LICENSE.GPL2
+** and LICENSE.GPL3 included in the packaging of this file.
+** Alternatively you may (at your option) use any later version
+** of the GNU General Public License if such license has been
+** publicly approved by Trolltech ASA (or its successors, if any)
+** and the KDE Free Qt Foundation.
 **
-** This file may be distributed and/or modified under the terms of the
-** GNU General Public License version 2 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/.
+** If you are unsure which license is appropriate for your use, please
+** review the following information:
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
 **
-** Licensees holding valid Qt Enterprise Edition licenses may use this
-** file in accordance with the Qt Commercial License Agreement provided
-** with the Software.
+** This file may be used under the terms of the Q Public License as
+** defined by Trolltech ASA and appearing in the file LICENSE.QPL
+** included in the packaging of this file.  Licensees holding valid Qt
+** Commercial licenses may use this file in accordance with the Qt
+** Commercial License Agreement provided with the Software.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-**
-** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
-**   information about Qt Commercial License Agreements.
-** See http://www.trolltech.com/qpl/ for QPL licensing information.
-** See http://www.trolltech.com/gpl/ for GPL licensing information.
-**
-** Contact info@trolltech.com if any conditions of this licensing are
-** not clear to you.
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
+** herein.
 **
 **********************************************************************/
 
@@ -53,7 +56,6 @@ static QGLFormat* qgl_default_overlay_format = 0;
 #undef INT32
 #undef INT8
 #include "qgl_x11_p.h"
-static void *gl_pixmap_visual = 0;
 #endif
 
 static QCleanupHandler<QGLFormat> qgl_cleanup_format;
@@ -1821,76 +1823,21 @@ QPixmap QGLWidget::renderPixmap( int w, int h, bool useContext )
     QSize sz = size();
     if ( (w > 0) && (h > 0) )
 	sz = QSize( w, h );
-    QPixmap pm;
-    pm.resize( sz );
 
 #if defined(Q_WS_X11)
-    // If we are using OpenGL widgets we HAVE to make sure that
-    // the default visual is GL enabled, otherwise it will wreck
-    // havock when e.g trying to render to GLXPixmaps via
-    // QPixmap. This is because QPixmap is always created with a
-    // QPaintDevice that uses x_appvisual per default. Preferably,
-    // use a visual that has depth and stencil buffers.
+    QPixmap pm( sz.width(), sz.height(), x11Depth() );
+    bool needConversion = x11Visual() != QPaintDevice::x11AppVisual();
 
-    if (!gl_pixmap_visual) {
-	int nvis;
-	Visual *vis = (Visual *) QPaintDevice::x11AppVisual();
-	int screen = QPaintDevice::x11AppScreen();
-	Display *appDpy = QPaintDevice::x11AppDisplay();
-	XVisualInfo * vi;
-	XVisualInfo visInfo;
-	memset( &visInfo, 0, sizeof(XVisualInfo) );
-	visInfo.visualid = XVisualIDFromVisual( vis );
-	visInfo.screen = screen;
-	vi = XGetVisualInfo( appDpy, VisualIDMask | VisualScreenMask, &visInfo, &nvis );
-	if ( vi ) {
-	    int useGL;
-	    int ret = glXGetConfig( appDpy, vi, GLX_USE_GL, &useGL );
-	    if ( ret != 0 || !useGL ) {
-		// We have to find another visual that is GL capable
-		int i;
-		XVisualInfo * visuals;
-		memset( &visInfo, 0, sizeof(XVisualInfo) );
-		visInfo.screen = screen;
-		visInfo.c_class = vi->c_class;
-		visInfo.depth = vi->depth;
-		visuals = XGetVisualInfo( appDpy, VisualClassMask |
-					  VisualDepthMask |
-					  VisualScreenMask, &visInfo,
-					  &nvis );
-		if ( visuals ) {
-		    for ( i = 0; i < nvis; i++ ) {
-			int ret = glXGetConfig( appDpy, &visuals[i], GLX_USE_GL, &useGL );
-			if ( ret == 0 && useGL ) {
-			    vis = visuals[i].visual;
-			    break;
-			}
-		    }
-		    XFree( visuals );
-		}
-	    }
-	    XFree( vi );
-	}
-	gl_pixmap_visual = vis;
+    // make sure the pixmap uses the same visual as the widget itself
+    if ( needConversion ) {
+        QPaintDeviceX11Data* xd = pm.getX11Data( TRUE );
+        xd->x_depth = x11Depth();
+        xd->x_visual = (Visual *) x11Visual();
+        pm.setX11Data( xd );
     }
-
-    if (gl_pixmap_visual != QPaintDevice::x11AppVisual()) {
-	int nvis = 0;
-	XVisualInfo visInfo;
-	memset( &visInfo, 0, sizeof(XVisualInfo) );
-	visInfo.visualid = XVisualIDFromVisual( (Visual *) gl_pixmap_visual );
-	visInfo.screen = QPaintDevice::x11AppScreen();
-	XVisualInfo *vi = XGetVisualInfo( QPaintDevice::x11AppDisplay(), VisualIDMask | VisualScreenMask,
-					  &visInfo, &nvis );
-	if (vi) {
-	    QPaintDeviceX11Data* xd = pm.getX11Data( TRUE );
-	    xd->x_depth = vi->depth;
-	    xd->x_visual = (Visual *) gl_pixmap_visual;
-	    pm.setX11Data( xd );
-	    XFree(vi);
-	}
-    }
-
+#else
+    QPixmap pm;
+    pm.resize( sz );
 #endif
 
     glcx->doneCurrent();
@@ -1923,7 +1870,7 @@ QPixmap QGLWidget::renderPixmap( int w, int h, bool useContext )
 
     if ( success ) {
 #if defined(Q_WS_X11)
-	if (gl_pixmap_visual != QPaintDevice::x11AppVisual()) {
+	if ( needConversion ) {
 	    QImage image = pm.convertToImage();
 	    QPixmap p;
 	    p = image;
@@ -2062,7 +2009,9 @@ void QGLWidget::qglColor( const QColor& c ) const
     if ( ctx ) {
 	if ( ctx->format().rgba() )
 	    glColor3ub( c.red(), c.green(), c.blue() );
-	else if ( !cmap.isEmpty() ) { // QGLColormap in use?
+
+	else if ( ctx->device() == context()->device()
+                  && !cmap.isEmpty() ) { // QGLColormap in use?
 	    int i = cmap.find( c.rgb() );
 	    if ( i < 0 )
 		i = cmap.findNearest( c.rgb() );
@@ -2087,7 +2036,8 @@ void QGLWidget::qglClearColor( const QColor& c ) const
 	if ( ctx->format().rgba() )
 	    glClearColor( (GLfloat)c.red() / 255.0, (GLfloat)c.green() / 255.0,
 			  (GLfloat)c.blue() / 255.0, (GLfloat) 0.0 );
-	else if ( !cmap.isEmpty() ) { // QGLColormap in use?
+	else if ( ctx->device() == context()->device()
+                  && !cmap.isEmpty() ) { // QGLColormap in use?
 	    int i = cmap.find( c.rgb() );
 	    if ( i < 0 )
 		i = cmap.findNearest( c.rgb() );
@@ -2248,10 +2198,14 @@ int QGLWidget::displayListBase( const QFont & fnt, int listBase )
    follow the last generated list. You would normally not have to
    change this value unless you are using lists in the same range. The
    lists are deleted when the widget is destroyed.
+
+   Note: This function only works reliably with ASCII strings.
 */
 
 void QGLWidget::renderText( int x, int y, const QString & str, const QFont & fnt, int listBase )
 {
+    if (str.isEmpty())
+        return;
     makeCurrent();
     glPushAttrib( GL_TRANSFORM_BIT | GL_VIEWPORT_BIT | GL_LIST_BIT | GL_CURRENT_BIT );
     glMatrixMode( GL_PROJECTION );
@@ -2265,7 +2219,8 @@ void QGLWidget::renderText( int x, int y, const QString & str, const QFont & fnt
     glRasterPos2i( 0, 0 );
     glBitmap(0, 0, 0, 0, x, -y, NULL);
     glListBase( displayListBase( fnt, listBase ) );
-    glCallLists( str.length(), GL_UNSIGNED_BYTE, str.local8Bit().data() );
+    const char *cstr = str.latin1();
+    glCallLists( qstrlen(cstr), GL_UNSIGNED_BYTE, cstr );
 
     // restore the matrix stacks and GL state
     glPopMatrix();
@@ -2284,11 +2239,14 @@ void QGLWidget::renderText( int x, int y, const QString & str, const QFont & fnt
 void QGLWidget::renderText( double x, double y, double z, const QString & str, const QFont & fnt,
 			    int listBase )
 {
+    if (str.isEmpty())
+        return;
     makeCurrent();
     glRasterPos3d( x, y, z );
     glPushAttrib( GL_LIST_BIT );
     glListBase( displayListBase( fnt, listBase ) );
-    glCallLists( str.length(), GL_UNSIGNED_BYTE, str.local8Bit().data() );
+    const char *cstr = str.latin1();
+    glCallLists( qstrlen(cstr), GL_UNSIGNED_BYTE, cstr );
     glPopAttrib();
 }
 
