@@ -2,7 +2,7 @@
 **
 ** Implementation of Interbase driver classes.
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the sql module of the Qt GUI Toolkit.
 ** EDITIONS: FREE, ENTERPRISE
@@ -496,7 +496,7 @@ bool QIBaseResult::exec()
 	    case SQL_INT64:
 		if (d->inda->sqlvar[para].sqlscale < 0)
 		    *((Q_LLONG*)d->inda->sqlvar[para].sqldata) = Q_LLONG(val.toDouble() * 
-									 pow(10, d->inda->sqlvar[para].sqlscale * -1));
+									 pow(10.0, d->inda->sqlvar[para].sqlscale * -1));
 		else
 		    *((Q_LLONG*)d->inda->sqlvar[para].sqldata) = val.toLongLong();
 		break;
@@ -650,7 +650,7 @@ bool QIBaseResult::gotoNext(QtSqlCachedResult::RowCache* row)
 	    break;
 	case SQL_INT64:
 	    if (d->sqlda->sqlvar[i].sqlscale < 0)
-		(*row)[i] = *(Q_LLONG*)buf * pow(10, d->sqlda->sqlvar[i].sqlscale);
+		(*row)[i] = *(Q_LLONG*)buf * pow(10.0, d->sqlda->sqlvar[i].sqlscale);
 	    else
 		(*row)[i] = QVariant(*(Q_LLONG*)buf);
 	    break;
