@@ -1,16 +1,16 @@
 /****************************************************************************
-** $Id: qt/qfile_unix.cpp   3.2.3   edited Oct 3 09:32 $
+** $Id: qt/qfile_unix.cpp   3.3.8   edited Jan 11 14:38 $
 **
 ** Implementation of QFile class
 **
 ** Created : 950628
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
 ** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
+** as defined by Trolltech ASA of Norway and appearing in the file
 ** LICENSE.QPL included in the packaging of this file.
 **
 ** This file may be distributed and/or modified under the terms of the
@@ -275,7 +275,11 @@ bool QFile::open( int m )
 	    length = INT_MAX;
 	    ioIndex = 0;
 	} else {
+#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_QT4)
+	    length = st.st_size > UINT_MAX ? UINT_MAX : (Offset)st.st_size;
+#else
 	    length = (Offset)st.st_size;
+#endif
 	    ioIndex = (flags() & IO_Append) == 0 ? 0 : length;
 	    if ( !(flags()&IO_Truncate) && length == 0 && isReadable() ) {
 		// try if you can read from it (if you can, it's a sequential
@@ -345,7 +349,12 @@ bool QFile::open( int m, FILE *f )
     struct stat st;
     ::fstat( fileno(fh), &st );
 #if defined(QT_LARGEFILE_SUPPORT)
+#if !defined(QT_ABI_QT4)
+    off_t tmp = ftello( fh );
+    ioIndex = tmp > UINT_MAX ? UINT_MAX : (Offset)tmp;
+#else
     ioIndex = (Offset)ftello( fh );
+#endif
 #else
     ioIndex = (Offset)ftell( fh );
 #endif
@@ -355,7 +364,11 @@ bool QFile::open( int m, FILE *f )
 	length = INT_MAX;
 	ioIndex = 0;
     } else {
+#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_QT4)
+	length = st.st_size > UINT_MAX ? UINT_MAX : (Offset)st.st_size;
+#else
 	length = (Offset)st.st_size;
+#endif
 	if ( !(flags()&IO_Truncate) && length == 0 && isReadable() ) {
 	    // try if you can read from it (if you can, it's a sequential
 	    // device; e.g. a file in the /proc filesystem)
@@ -407,14 +420,23 @@ bool QFile::open( int m, int f )
     ext_f = TRUE;
     struct stat st;
     ::fstat( fd, &st );
+#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_QT4)
+    off_t tmp = ::lseek(fd, 0, SEEK_CUR);
+    ioIndex = tmp > UINT_MAX ? UINT_MAX : (Offset)tmp;
+#else
     ioIndex = (Offset)::lseek(fd, 0, SEEK_CUR);
+#endif
     if ( (st.st_mode & S_IFMT) != S_IFREG || f == 0 ) { // stdin is not seekable...
 	// non-seekable
 	setType( IO_Sequential );
 	length = INT_MAX;
 	ioIndex = 0;
     } else {
+#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_QT4)
+	length = st.st_size > UINT_MAX ? UINT_MAX : (Offset)st.st_size;
+#else
 	length = (Offset)st.st_size;
+#endif
 	if ( length == 0 && isReadable() ) {
 	    // try if you can read from it (if you can, it's a sequential
 	    // device; e.g. a file in the /proc filesystem)
@@ -447,8 +469,8 @@ QIODevice::Offset QFile::size() const
     }
     if ( ret == -1 )
 	return 0;
-#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_64BITOFFSET)
-    return (uint)st.st_size > UINT_MAX ? UINT_MAX : (QIODevice::Offset)st.st_size;
+#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_QT4)
+    return (uint)st.st_size > UINT_MAX ? UINT_MAX : (Offset)st.st_size;
 #else
     return st.st_size;
 #endif
@@ -502,11 +524,15 @@ bool QFile::at( Offset pos )
 #endif
     }
     if ( ok )
-	ioIndex = pos;
+#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_QT4)
+	ioIndex = pos > UINT_MAX ? UINT_MAX : (Offset)pos;
+#else
+	ioIndex = (Offset)pos;
+#endif
 #if defined(QT_CHECK_RANGE)
     else
-#if defined(QT_LARGEFILE_SUPPORT) && defined(QT_ABI_64BITOFFSET)
-	qWarning( "QFile::at: Cannot set file position %llu", pos );
+#if defined(QT_ABI_QT4)
+	qWarning( "QFile::at: Cannot set file position %lld", pos );
 #else
 	qWarning( "QFile::at: Cannot set file position %lu", pos );
 #endif
@@ -624,14 +650,25 @@ Q_LONG QFile::writeBlock( const char *p, Q_ULONG len )
 	    setStatus( IO_WriteError );
 	setErrorStringErrno( errno );
 	if ( !isSequentialAccess() ) {
-	    if ( isRaw() )			// recalc file position
+	    if ( isRaw() ) {			// recalc file position
+#if defined(QT_LARGEFILE_SUPPORT) && !defined(QT_ABI_QT4)
+		off_t tmp = ::lseek( fd, 0, SEEK_CUR );
+		ioIndex = tmp > UINT_MAX ? UINT_MAX : (Offset)tmp;
+#else
 		ioIndex = (Offset)::lseek( fd, 0, SEEK_CUR );
-	    else
+#endif
+	    } else {
 #if defined(QT_LARGEFILE_SUPPORT)
+#if !defined(QT_ABI_QT4)
+		off_t tmp = (Offset)::fseeko( fh, 0, SEEK_CUR );
+		ioIndex = tmp > UINT_MAX ? UINT_MAX : (Offset)tmp;
+#else
 		ioIndex = (Offset)::fseeko( fh, 0, SEEK_CUR );
+#endif
 #else
 		ioIndex = (Offset)::fseek( fh, 0, SEEK_CUR );
 #endif
+	    }
 	}
     } else {
 	if ( !isSequentialAccess() )
