@@ -1,16 +1,16 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qintcache.h   3.3.8   edited Jan 11 14:38 $
 **
 ** Definition of QIntCache template class
 **
 ** Created : 950209
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
 ** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
+** as defined by Trolltech ASA of Norway and appearing in the file
 ** LICENSE.QPL included in the packaging of this file.
 **
 ** This file may be distributed and/or modified under the terms of the
@@ -45,7 +45,11 @@
 
 template<class type> 
 class QIntCache
-: public QGCache
+#ifdef Q_QDOC
+	: public QPtrCollection
+#else
+	: public QGCache
+#endif
 {
 public:
     QIntCache( const QIntCache<type> &c ) : QGCache(c) {}
@@ -88,8 +92,7 @@ template<class type> inline void QIntCache<type>::deleteItem( QPtrCollection::It
 }
 
 template<class type> 
-class QIntCacheIterator
-: public QGCacheIterator
+class QIntCacheIterator : public QGCacheIterator
 {
 public:
     QIntCacheIterator( const QIntCache<type> &c )
