@@ -1,16 +1,16 @@
 /****************************************************************************
-** $Id: qt/qtabwidget.cpp   3.2.3   edited Jun 27 09:25 $
+** $Id: qt/qtabwidget.cpp   3.3.8   edited Jan 11 14:39 $
 **
 ** Implementation of QTabWidget class
 **
 ** Created : 990318
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2007 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
 ** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
+** as defined by Trolltech ASA of Norway and appearing in the file
 ** LICENSE.QPL included in the packaging of this file.
 **
 ** This file may be distributed and/or modified under the terms of the
@@ -47,6 +47,10 @@
 #include "qpainter.h"
 #include "qtoolbutton.h"
 
+#ifdef Q_OS_MACX
+#include <qmacstyle_mac.h>
+#endif
+
 /*!
     \class QTabWidget qtabwidget.h
     \brief The QTabWidget class provides a stack of tabbed widgets.
@@ -226,12 +230,19 @@ QTabWidget::QTabWidget( QWidget *parent, const char *name, WFlags f )
     setTabBar( new QTabBar( this, "tab control" ) );
 
     d->stack->setFrameStyle( QFrame::TabWidgetPanel | QFrame::Raised );
+#ifdef Q_OS_TEMP
+    d->pos = Bottom;
+#endif
 
     setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
     setFocusPolicy( TabFocus );
     setFocusProxy( d->tabs );
 
     installEventFilter( this );
+#ifdef Q_OS_MACX
+    if (::qt_cast<QMacStyle*>(&style()))
+        setMargin(10); // According to HIGuidelines at least.
+#endif
 }
 
 /*!
@@ -260,6 +271,8 @@ QTabWidget::~QTabWidget()
     If you call addTab() after show() the screen will flicker and the
     user may be confused.
 
+    Adding the same child twice will have undefined behavior.
+
     \sa insertTab()
 */
 void QTabWidget::addTab( QWidget *child, const QString &label)
@@ -380,15 +393,7 @@ void QTabWidget::changeTab( QWidget *w, const QString &label)
     // this will update the accelerators
     t->setText( label );
 
-    int ct = d->tabs->currentTab();
-    bool block = d->tabs->signalsBlocked();
-    d->tabs->blockSignals( TRUE );
-    QTab* ft = d->tabs->tabAt( 0 );
-    if ( ft )
-	d->tabs->setCurrentTab( ft->identifier() );
-    d->tabs->setCurrentTab( ct );
-    d->tabs->blockSignals( block );
-
+    d->tabs->layoutTabs();
     d->tabs->update();
     setUpLayout();
 }
@@ -414,15 +419,7 @@ void QTabWidget::changeTab( QWidget *w, const QIconSet& iconset, const QString &
     t->iconset = new QIconSet( iconset );
     t->setText( label );
 
-    int ct = d->tabs->currentTab();
-    bool block = d->tabs->signalsBlocked();
-    d->tabs->blockSignals( TRUE );
-    QTab* ft = d->tabs->tabAt( 0 );
-    if ( ft )
-	d->tabs->setCurrentTab( ft->identifier() );
-    d->tabs->setCurrentTab( ct );
-    d->tabs->blockSignals( block );
-
+    d->tabs->layoutTabs();
     d->tabs->update();
     setUpLayout();
 }
@@ -684,13 +681,17 @@ void QTabWidget::setUpLayout( bool onlyCheck )
 	t = d->tabs->sizeHint();
     int lcw = 0;
     if ( d->leftCornerWidget && d->leftCornerWidget->isVisible()  ) {
-	lcw = d->leftCornerWidget->width();
+	QSize sz = d->leftCornerWidget->sizeHint();
+	d->leftCornerWidget->resize(sz);
+	lcw = sz.width();
 	if ( t.height() > lcw )
 	    lcw = t.height();
      }
     int rcw = 0;
     if ( d->rightCornerWidget && d->rightCornerWidget->isVisible() ) {
-	rcw = d->rightCornerWidget->width();
+	QSize sz = d->rightCornerWidget->sizeHint();
+	d->rightCornerWidget->resize(sz);
+	rcw = sz.width();
 	if ( t.height() > rcw )
 	    rcw = t.height();
     }
@@ -704,11 +705,10 @@ void QTabWidget::setUpLayout( bool onlyCheck )
     exth = style().pixelMetric( QStyle::PM_TabBarBaseHeight, this );
     overlap = style().pixelMetric( QStyle::PM_TabBarBaseOverlap, this );
 
-    if ( reverse ) {
+    if ( reverse )
 	tabx = QMIN( width() - t.width(), width() - t.width() - lw + 2 ) - lcw;
-    } else {
+    else
 	tabx = QMAX( 0, lw - 2 ) + lcw;
-    }
     if ( d->pos == Bottom ) {
 	taby = height() - t.height() - lw;
 	stacky = 0;
@@ -723,9 +723,9 @@ void QTabWidget::setUpLayout( bool onlyCheck )
     int alignment = style().styleHint( QStyle::SH_TabBar_Alignment, this );
     if ( alignment != AlignLeft && t.width() < width() ) {
 	if ( alignment == AlignHCenter )
-	    tabx += width()/2 - t.width()/2;
+	    tabx += (width()-lcw-rcw)/2 - t.width()/2;
 	else if ( alignment == AlignRight )
-	    tabx += width() - t.width();
+	    tabx += width() - t.width() - rcw;
     }
 
     d->tabs->setGeometry( tabx, taby, t.width(), t.height() );
@@ -739,10 +739,6 @@ void QTabWidget::setUpLayout( bool onlyCheck )
 			   t.height()+QMAX(0, lw-2));
 
     d->dirty = FALSE;
-    if ( !onlyCheck )
-	update();
-    if ( autoMask() )
-	updateMask();
 
     // move cornerwidgets
     if ( d->leftCornerWidget ) {
@@ -754,7 +750,12 @@ void QTabWidget::setUpLayout( bool onlyCheck )
 	int y = ( t.height() / 2 ) - ( d->rightCornerWidget->height() / 2 );
 	int x = ( reverse ? y : width() - rcw + y );
 	d->rightCornerWidget->move( x, y + taby );
-    }
+    }    
+    if ( !onlyCheck )
+	update();
+    updateGeometry();
+    if ( autoMask() )
+	updateMask();
 }
 
 /*!
@@ -762,15 +763,25 @@ void QTabWidget::setUpLayout( bool onlyCheck )
 */
 QSize QTabWidget::sizeHint() const
 {
+    QSize lc(0, 0), rc(0, 0);
+
+    if (d->leftCornerWidget)
+	lc = d->leftCornerWidget->sizeHint();
+    if(d->rightCornerWidget)
+	rc = d->rightCornerWidget->sizeHint();
     if ( !d->dirty ) {
 	QTabWidget *that = (QTabWidget*)this;
 	that->setUpLayout( TRUE );
     }
     QSize s( d->stack->sizeHint() );
-    QSize t( d->tabs->sizeHint().boundedTo( QSize(200,200) ) );
+    QSize t( d->tabs->sizeHint() );
+    if(!style().styleHint(QStyle::SH_TabBar_PreferNoArrows, d->tabs))
+	t = t.boundedTo( QSize(200,200) );
+    else
+        t = t.boundedTo( QApplication::desktop()->size() );
 
-    QSize sz( QMAX( s.width(), t.width() ), s.height() + t.height() +
-	      ( d->tabBase->isVisible() ? d->tabBase->height() : 0 ) );
+    QSize sz( QMAX( s.width(), t.width() + rc.width() + lc.width() ),
+	      s.height() + (QMAX( rc.height(), QMAX(lc.height(), t.height()))) + ( d->tabBase->isVisible() ? d->tabBase->height() : 0 ) );
     return style().sizeFromContents(QStyle::CT_TabWidget, this, sz).expandedTo(QApplication::globalStrut());
 }
 
@@ -782,14 +793,21 @@ QSize QTabWidget::sizeHint() const
 */
 QSize QTabWidget::minimumSizeHint() const
 {
+    QSize lc(0, 0), rc(0, 0);
+
+    if(d->leftCornerWidget)
+	lc = d->leftCornerWidget->minimumSizeHint();
+    if(d->rightCornerWidget)
+	rc = d->rightCornerWidget->minimumSizeHint();
     if ( !d->dirty ) {
 	QTabWidget *that = (QTabWidget*)this;
 	that->setUpLayout( TRUE );
     }
     QSize s( d->stack->minimumSizeHint() );
     QSize t( d->tabs->minimumSizeHint() );
-    QSize sz( QMAX( s.width(), t.width() ), s.height() + t.height() +
-	      ( d->tabBase->isVisible() ? d->tabBase->height() : 0 ) );
+
+    QSize sz( QMAX( s.width(), t.width() + rc.width() + lc.width() ),
+	      s.height() + (QMAX( rc.height(), QMAX(lc.height(), t.height()))) + ( d->tabBase->isVisible() ? d->tabBase->height() : 0 ) );
     return style().sizeFromContents(QStyle::CT_TabWidget, this, sz).expandedTo(QApplication::globalStrut());
 }
 
@@ -1032,16 +1050,6 @@ void QTabWidget::setTabIconSet( QWidget * w, const QIconSet & iconset )
     t->iconset = new QIconSet( iconset );
 
     d->tabs->layoutTabs();
-
-    int ct = d->tabs->currentTab();
-    bool block = d->tabs->signalsBlocked();
-    d->tabs->blockSignals( TRUE );
-    QTab* ft = d->tabs->tabAt( 0 );
-    if ( ft )
-	d->tabs->setCurrentTab( ft->identifier() );
-    d->tabs->setCurrentTab( ct );
-    d->tabs->blockSignals( block );
-
     d->tabs->update();
     setUpLayout();
 }
