@@ -1,37 +1,40 @@
 /****************************************************************************
-** $Id: qt/qtoolbar.cpp   3.2.3   edited Jul 4 11:53 $
 **
 ** Implementation of QToolBar class
 **
 ** Created : 980315
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2008 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
-** This file may be distributed under the terms of the Q Public License
-** as defined by Trolltech AS of Norway and appearing in the file
-** LICENSE.QPL included in the packaging of this file.
+** This file may be used under the terms of the GNU General
+** Public License versions 2.0 or 3.0 as published by the Free
+** Software Foundation and appearing in the files LICENSE.GPL2
+** and LICENSE.GPL3 included in the packaging of this file.
+** Alternatively you may (at your option) use any later version
+** of the GNU General Public License if such license has been
+** publicly approved by Trolltech ASA (or its successors, if any)
+** and the KDE Free Qt Foundation.
 **
-** This file may be distributed and/or modified under the terms of the
-** GNU General Public License version 2 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.
+** Please review the following information to ensure GNU General
+** Public Licensing requirements will be met:
+** http://trolltech.com/products/qt/licenses/licensing/opensource/.
+** If you are unsure which license is appropriate for your use, please
+** review the following information:
+** http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+** or contact the sales department at sales@trolltech.com.
 **
-** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
-** licenses may use this file in accordance with the Qt Commercial License
-** Agreement provided with the Software.
+** This file may be used under the terms of the Q Public License as
+** defined by Trolltech ASA and appearing in the file LICENSE.QPL
+** included in the packaging of this file.  Licensees holding valid Qt
+** Commercial licenses may use this file in accordance with the Qt
+** Commercial License Agreement provided with the Software.
 **
-** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-**
-** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
-**   information about Qt Commercial License Agreements.
-** See http://www.trolltech.com/qpl/ for QPL licensing information.
-** See http://www.trolltech.com/gpl/ for GPL licensing information.
-**
-** Contact info@trolltech.com if any conditions of this licensing are
-** not clear to you.
+** This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+** INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+** A PARTICULAR PURPOSE. Trolltech reserves all rights not granted
+** herein.
 **
 **********************************************************************/
 
@@ -39,7 +42,6 @@
 #ifndef QT_NO_TOOLBAR
 
 #include "qmainwindow.h"
-#include "qpushbutton.h"
 #include "qtooltip.h"
 #include "qcursor.h"
 #include "qlayout.h"
@@ -88,13 +90,11 @@ class QToolBarPrivate
 {
 public:
     QToolBarPrivate() : moving( FALSE ) {
-	extensionSubMenues.setAutoDelete(TRUE);
     }
 
     bool moving;
     QToolBarExtensionWidget *extension;
     QPopupMenu *extensionPopup;
-    QObjectList extensionSubMenues;
 };
 
 
@@ -375,18 +375,14 @@ QToolBar::QToolBar( QMainWindow * parent, const char * name )
 void QToolBar::init()
 {
     d = new QToolBarPrivate;
-    d->extension = new QToolBarExtensionWidget( this );
-    d->extension->hide();
-    d->extensionPopup = new QPopupMenu( this, "qt_dockwidget_internal" );
-    connect( d->extensionPopup, SIGNAL( aboutToShow() ),
-	     this, SLOT( createPopup() ) );
-    d->extension->button()->setPopup( d->extensionPopup );
-    d->extension->button()->setPopupDelay( -1 );
+    d->extension = 0;
+    d->extensionPopup = 0;
     sw = 0;
 
     setBackgroundMode( PaletteButton);
     setFocusPolicy( NoFocus );
     setFrameStyle( QFrame::ToolBarPanel | QFrame::Raised);
+    boxLayout()->setSpacing(style().pixelMetric(QStyle::PM_ToolBarItemSpacing));
 }
 
 /*!
@@ -406,7 +402,8 @@ QToolBar::~QToolBar()
 void QToolBar::setOrientation( Orientation o )
 {
     QDockWindow::setOrientation( o );
-    d->extension->setOrientation( o );
+    if (d->extension)
+	d->extension->setOrientation( o );
     QObjectList *childs = queryList( "QToolBarSeparator" );
     if ( childs ) {
         QObject *ob = 0;
@@ -443,6 +440,7 @@ void QToolBar::styleChange( QStyle& )
         }
     }
     delete childs;
+    boxLayout()->setSpacing(style().pixelMetric(QStyle::PM_ToolBarItemSpacing));
 }
 
 /*!
@@ -602,10 +600,25 @@ QSize QToolBar::minimumSizeHint() const
 
 void QToolBar::createPopup()
 {
-    d->extensionPopup->clear();
-    d->extensionSubMenues.clear();
+    if (!d->extensionPopup) {
+	d->extensionPopup = new QPopupMenu( this, "qt_dockwidget_internal" );
+	connect( d->extensionPopup, SIGNAL( aboutToShow() ), this, SLOT( createPopup() ) );
+    }
 
-    QObjectList *childlist = queryList( "QWidget", 0, FALSE, TRUE );
+    if (!d->extension) {
+	d->extension = new QToolBarExtensionWidget( this );
+	d->extension->setOrientation(orientation());
+	d->extension->button()->setPopup( d->extensionPopup );
+	d->extension->button()->setPopupDelay( -1 );
+    }
+
+    d->extensionPopup->clear();
+    // clear doesn't delete submenus, so do this explicitly
+    QObjectList *childlist = d->extensionPopup->queryList( "QPopupMenu", 0, FALSE, TRUE );
+    childlist->setAutoDelete(TRUE);
+    delete childlist;
+
+    childlist = queryList( "QWidget", 0, FALSE, TRUE );
     QObjectListIt it( *childlist );
     bool hide = FALSE;
     bool doHide = FALSE;
@@ -638,7 +651,7 @@ void QToolBar::createPopup()
 	        QString s = b->textLabel();
 	        if ( s.isEmpty() )
 		    s = b->text();
-	        if ( b->popup() && b->popupDelay() == 0 )
+	        if ( b->popup() && b->popupDelay() <= 0 )
 		    id = d->extensionPopup->insertItem( b->iconSet(), s, b->popup() );
 		else
 		    id = d->extensionPopup->insertItem( b->iconSet(), s, b, SLOT( emulateClick() ) ) ;
@@ -671,11 +684,10 @@ void QToolBar::createPopup()
 		    if ( s.isEmpty() )
 		        s = c->currentText();
 		    uint maxItems = 0;
-		    QPopupMenu *cp = new QPopupMenu;
-		    d->extensionSubMenues.append( cp );
+		    QPopupMenu *cp = new QPopupMenu(d->extensionPopup);
+                    cp->setEnabled(c->isEnabled());
 		    d->extensionPopup->insertItem( s, cp );
-		    connect( cp, SIGNAL( activated( int ) ),
-		        c, SLOT( internalActivate( int ) ) );
+		    connect( cp, SIGNAL( activated(int) ), c, SLOT( internalActivate(int) ) );
 		    for ( int i = 0; i < c->count(); ++i ) {
 		        QString tmp = c->text( i );
 			cp->insertItem( tmp, i );
@@ -687,12 +699,10 @@ void QToolBar::createPopup()
 			        maxItems = QApplication::desktop()->height() * 10 / h;
 			    }
 			} else if ( cp->count() >= maxItems - 1 ) {
-			    QPopupMenu* sp = new QPopupMenu;
-			    d->extensionSubMenues.append( sp );
+			    QPopupMenu* sp = new QPopupMenu(d->extensionPopup);
 			    cp->insertItem( tr( "More..." ), sp );
 			    cp = sp;
-			    connect( cp, SIGNAL( activated( int ) ),
-			        c, SLOT( internalActivate( int ) ) );
+			    connect( cp, SIGNAL( activated(int) ), c, SLOT( internalActivate(int) ) );
 			}
 		    }
 		}
@@ -716,6 +726,9 @@ void QToolBar::resizeEvent( QResizeEvent *e )
 
 void QToolBar::checkForExtension( const QSize &sz )
 {
+    if (!isVisible())
+	return;
+
     bool tooSmall;
     if ( orientation() == Horizontal )
 	tooSmall = sz.width() < sizeHint().width();
@@ -732,10 +745,16 @@ void QToolBar::checkForExtension( const QSize &sz )
 	    d->extension->show();
 	    d->extension->raise();
 	} else {
-	    d->extension->hide();
+	    delete d->extension;
+	    d->extension = 0;
+	    delete d->extensionPopup;
+	    d->extensionPopup = 0;
 	}
     } else {
-	d->extension->hide();
+	delete d->extension;
+	d->extension = 0;
+	delete d->extensionPopup;
+	d->extensionPopup = 0;
     }
 }
 
