@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2007 Trolltech ASA.  All rights reserved.
 **
 ** This file is part of Qt Linguist.
 **
@@ -293,10 +293,7 @@ TrWindow::TrWindow()
     foundWhere = 0;
     foundOffset = 0;
 
-    connect( lv, SIGNAL(currentChanged(QListViewItem *)),
-	     this, SLOT(showNewScope(QListViewItem *)) );
-
-    connect( lv, SIGNAL(clicked(QListViewItem *, const QPoint&, int)),
+    connect( lv, SIGNAL(selectionChanged(QListViewItem *)),
 	     this, SLOT(showNewScope(QListViewItem *)) );
 
     connect( slv, SIGNAL(currentChanged(QListViewItem *)),
@@ -773,7 +770,7 @@ void TrWindow::openPhraseBook()
 	tr("Open Phrase Book") );
     if ( !name.isEmpty() && !phraseBookNames.contains(name) ) {
 	if ( openPhraseBook(name) ) {
-	    int n = phraseBooks.at( phraseBooks.count() - 1 )->count();
+	    int n = (int)(phraseBooks.at( phraseBooks.count() - 1 )->count());
 	    statusBar()->message( tr("%1 phrase(s) loaded.").arg(n),
 				  MessageMS );
 	}
@@ -894,7 +891,7 @@ bool TrWindow::maybeSave()
 		return FALSE;
 	    case QMessageBox::Yes:
 		save();
-		break;
+                return !dirty;
 	    case QMessageBox::No:
 		break;
 	}
@@ -951,6 +948,10 @@ void TrWindow::showNewScope( QListViewItem *item )
 	    tmp->updateTranslationText();
 	}
 
+	// Make sure that we update the source text and translation text. Use the first message in the new scope.
+	setCurrentMessageItem(slv->firstChild());
+	// This must be explicitly called since the signal is not emitted if the context only contains one item
+	showNewCurrent(slv->firstChild());
 	slv->viewport()->setUpdatesEnabled( upe );
 	slv->setUpdatesEnabled( upe );
 	if( upe )
