@@ -3,107 +3,107 @@
     <name>QAccel</name>
     <message>
         <source>Space</source>
-        <translation>Space</translation>
+        <translation>Пробел</translation>
     </message>
     <message>
         <source>Esc</source>
-        <translation>Esc</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Tab</source>
-        <translation>Tab</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Backtab</source>
-        <translation>Backtab</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Backspace</source>
-        <translation>Backspace</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Return</source>
-        <translation>Return</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Enter</source>
-        <translation>Enter</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Ins</source>
-        <translation>Ins</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Del</source>
-        <translation>Del</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Pause</source>
-        <translation>Pause</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Print</source>
-        <translation>Print</translation>
+        <translation></translation>
     </message>
     <message>
         <source>SysReq</source>
-        <translation>SysReq</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Home</source>
-        <translation>Home</translation>
+        <translation></translation>
     </message>
     <message>
         <source>End</source>
-        <translation>End</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Left</source>
-        <translation>Left</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Up</source>
-        <translation>Up</translation>
+        <translation>Стрелка вверх</translation>
     </message>
     <message>
         <source>Right</source>
-        <translation>Right</translation>
+        <translation>Стрелка вправо</translation>
     </message>
     <message>
         <source>Down</source>
-        <translation>Down</translation>
+        <translation>Стрелка вниз</translation>
     </message>
     <message>
         <source>PgUp</source>
-        <translation>PgUp</translation>
+        <translation></translation>
     </message>
     <message>
         <source>PgDown</source>
-        <translation>PgDown</translation>
+        <translation></translation>
     </message>
     <message>
         <source>CapsLock</source>
-        <translation>CapsLock</translation>
+        <translation></translation>
     </message>
     <message>
         <source>NumLock</source>
-        <translation>NumLock</translation>
+        <translation></translation>
     </message>
     <message>
         <source>ScrollLock</source>
-        <translation>ScrollLock</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Ctrl</source>
-        <translation>Ctrl</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Alt</source>
-        <translation>Alt</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Shift</source>
-        <translation>Shift</translation>
+        <translation></translation>
     </message>
     <message>
         <source>+</source>
@@ -111,63 +111,63 @@
     </message>
     <message>
         <source>F%1</source>
-        <translation>F%1</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Menu</source>
-        <translation>Меню</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Help</source>
-        <translation>Справка</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Back</source>
-        <translation>Назад</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Forward</source>
-        <translation>Вперед</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Stop</source>
-        <translation>Стоп</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Refresh</source>
-        <translation>Обновить</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Volume Down</source>
-        <translation>Тише</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Volume Mute</source>
-        <translation>Выключить звук</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Volume Up</source>
-        <translation>Громче</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Bass Boost</source>
-        <translation>Bass Boost</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Bass Up</source>
-        <translation>Bass Up</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Bass Down</source>
-        <translation>Bass Down</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Treble Up</source>
-        <translation>Treble Up</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Treble Down</source>
-        <translation>Treble Down</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Media Play</source>
@@ -199,47 +199,47 @@
     </message>
     <message>
         <source>Standby</source>
-        <translation>Дежурный режим</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Open URL</source>
-        <translation>Открыть URL</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch Mail</source>
-        <translation>Почта</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch Media</source>
-        <translation>Проигрыватель</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (0)</source>
-        <translation>Запустить (0)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (1)</source>
-        <translation>Запустить (1)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (2)</source>
-        <translation>Запустить (2)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (3)</source>
-        <translation>Запустить (3)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (4)</source>
-        <translation>Запустить (4)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (5)</source>
-        <translation>Запустить (5)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (6)</source>
-        <translation>Запустить (6)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (7)</source>
@@ -247,23 +247,23 @@
     </message>
     <message>
         <source>Launch (8)</source>
-        <translation>Запустить (8)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (9)</source>
-        <translation>Запустить (9)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (A)</source>
-        <translation>Запустить (A)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (B)</source>
-        <translation>Запустить (B)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (C)</source>
-        <translation>Запустить (C)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (D)</source>
@@ -271,15 +271,67 @@
     </message>
     <message>
         <source>Launch (E)</source>
-        <translation>Запустить (E)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Launch (F)</source>
-        <translation>Запустить (F)</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Meta</source>
-        <translation>Meta</translation>
+        <translation></translation>
+    </message>
+    <message>
+        <source>%1, %2 not defined</source>
+        <translation>%1, %2 не определено</translation>
+    </message>
+    <message>
+        <source>Ambiguous &quot;%1&quot; not handled</source>
+        <translation>Неоднозначное &quot;%1&quot; не поддерживается</translation>
+    </message>
+    <message>
+        <source>Print Screen</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Page Up</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Page Down</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Caps Lock</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Num Lock</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Number Lock</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Scroll Lock</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Insert</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Delete</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Escape</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>System Request</source>
+        <translation>Системный запрос</translation>
     </message>
 </context>
 <context>
@@ -291,7 +343,7 @@
     </message>
     <message>
         <source>Executable &apos;%1&apos; requires Qt %2, found Qt %3.</source>
-        <translation>Программный модуль &apos;%1&apos; требует Qt %2, найдена версия %3.</translation>
+        <translation>Программный модуль &apos;%1&apos; требует Qt %2, найдена версия Qt %3.</translation>
     </message>
     <message>
         <source>Incompatible Qt Library Error</source>
@@ -302,11 +354,11 @@
     <name>QAquaStyle</name>
     <message>
         <source>OK</source>
-        <translation>OK</translation>
+        <translation type="obsolete">OK</translation>
     </message>
     <message>
         <source>Cancel</source>
-        <translation>Отмена</translation>
+        <translation type="obsolete">Отмена</translation>
     </message>
 </context>
 <context>
@@ -317,23 +369,23 @@
     </message>
     <message>
         <source>&amp;Sat:</source>
-        <translation>&amp;Нас:</translation>
+        <translation>&amp;Насыщенность:</translation>
     </message>
     <message>
         <source>&amp;Val:</source>
-        <translation>&amp;Ярк:</translation>
+        <translation>&amp;Яркость:</translation>
     </message>
     <message>
         <source>&amp;Red:</source>
-        <translation>&amp;Крас:</translation>
+        <translation>&amp;Красный:</translation>
     </message>
     <message>
         <source>&amp;Green:</source>
-        <translation>&amp;Зел:</translation>
+        <translation>&amp;Зелёный:</translation>
     </message>
     <message>
         <source>Bl&amp;ue:</source>
-        <translation>С&amp;ин:</translation>
+        <translation>С&amp;иний:</translation>
     </message>
     <message>
         <source>A&amp;lpha channel:</source>
@@ -372,11 +424,11 @@
     <name>QDataTable</name>
     <message>
         <source>True</source>
-        <translation>True</translation>
+        <translation>Правда</translation>
     </message>
     <message>
         <source>False</source>
-        <translation>False</translation>
+        <translation>Ложь</translation>
     </message>
     <message>
         <source>Insert</source>
@@ -395,7 +447,70 @@
     <name>QDialog</name>
     <message>
         <source>What&apos;s This?</source>
-        <translation>Что это?</translation>
+        <translation>Что это такое?</translation>
+    </message>
+    <message>
+        <source>Help</source>
+        <translation>Помошь</translation>
+    </message>
+</context>
+<context>
+    <name>QDialogButtons</name>
+    <message>
+        <source>Yes to All</source>
+        <translation>Да для всех</translation>
+    </message>
+    <message>
+        <source>OK to All</source>
+        <translation>Ок для всех</translation>
+    </message>
+    <message>
+        <source>No to All</source>
+        <translation>Нет для всех</translation>
+    </message>
+    <message>
+        <source>Cancel All</source>
+        <translation>Отмена для всех</translation>
+    </message>
+    <message>
+        <source> to All</source>
+        <translation>для всех</translation>
+    </message>
+    <message>
+        <source>Yes</source>
+        <translation>Да</translation>
+    </message>
+    <message>
+        <source>OK</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>No</source>
+        <translation>Нет</translation>
+    </message>
+    <message>
+        <source>Cancel</source>
+        <translation>Отмена</translation>
+    </message>
+    <message>
+        <source>Apply</source>
+        <translation>Применить</translation>
+    </message>
+    <message>
+        <source>Ignore</source>
+        <translation>Игнорировать</translation>
+    </message>
+    <message>
+        <source>Retry</source>
+        <translation>Повторить</translation>
+    </message>
+    <message>
+        <source>Abort</source>
+        <translation>Прервать</translation>
+    </message>
+    <message>
+        <source>Help</source>
+        <translation>Помощь</translation>
     </message>
 </context>
 <context>
@@ -421,6 +536,37 @@
         <translation>Критическая ошибка:</translation>
     </message>
 </context>
+<context>
+    <name>QFile</name>
+    <message>
+        <source>Unknown error</source>
+        <translation>Неизвестная ошибка</translation>
+    </message>
+    <message>
+        <source>Could not read from the file</source>
+        <translation>Не удается прочитать из файла</translation>
+    </message>
+    <message>
+        <source>Could not write to the file</source>
+        <translation>Не удается записать в файл</translation>
+    </message>
+    <message>
+        <source>Permission denied</source>
+        <translation>Нет прав</translation>
+    </message>
+    <message>
+        <source>Too many open files</source>
+        <translation>Слишком много открытых файлов</translation>
+    </message>
+    <message>
+        <source>No such file or directory</source>
+        <translation>Именно такого файла или каталога нет</translation>
+    </message>
+    <message>
+        <source>No space left on device</source>
+        <translation>Нет места на диске</translation>
+    </message>
+</context>
 <context>
     <name>QFileDialog</name>
     <message>
@@ -465,11 +611,11 @@
     </message>
     <message>
         <source>OK</source>
-        <translation>OK</translation>
+        <translation type="obsolete">OK</translation>
     </message>
     <message>
         <source>Look &amp;in:</source>
-        <translation>&amp;Смотреть в:</translation>
+        <translation>&amp;Искать в:</translation>
     </message>
     <message>
         <source>File &amp;name:</source>
@@ -497,15 +643,15 @@
     </message>
     <message>
         <source>Detail View</source>
-        <translation>Детальный вид</translation>
+        <translation>Подробный вид</translation>
     </message>
     <message>
         <source>Preview File Info</source>
-        <translation>Предпросмотр информации о файле</translation>
+        <translation>Предварительный просмотр информации о файле</translation>
     </message>
     <message>
         <source>Preview File Contents</source>
-        <translation>Предпросмотр содержимого файла</translation>
+        <translation>Предварительный просмотр содержимого файла</translation>
     </message>
     <message>
         <source>Read-write</source>
@@ -609,7 +755,7 @@
     </message>
     <message>
         <source>the symlink</source>
-        <translation>ссылку</translation>
+        <translation>ссылка</translation>
     </message>
     <message>
         <source>Delete %1</source>
@@ -649,7 +795,7 @@
     </message>
     <message>
         <source>Save</source>
-        <translation>Сохранить</translation>
+        <translation type="obsolete">Сохранить</translation>
     </message>
     <message>
         <source>Error</source>
@@ -665,20 +811,24 @@ Check path and filename.</source>
     </message>
     <message>
         <source>All Files (*.*)</source>
-        <translation>Все файлы (*.*)</translation>
+        <translation type="obsolete">Все файлы (*.*)</translation>
     </message>
     <message>
         <source>Open </source>
-        <translation>Открыть</translation>
+        <translation type="obsolete">Открыть</translation>
     </message>
     <message>
         <source>Select a Directory</source>
-        <translation>Выбрать каталог</translation>
+        <translation type="obsolete">Выбрать каталог</translation>
     </message>
     <message>
         <source>Directory:</source>
         <translation>Каталог:</translation>
     </message>
+    <message>
+        <source>&amp;OK</source>
+        <translation>&amp;OK</translation>
+    </message>
 </context>
 <context>
     <name>QFont</name>
@@ -724,7 +874,7 @@ Check path and filename.</source>
     </message>
     <message>
         <source>Thaana</source>
-        <translation>Thaana</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Devanagari</source>
@@ -736,11 +886,11 @@ Check path and filename.</source>
     </message>
     <message>
         <source>Gurmukhi</source>
-        <translation>Gurmukhi</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Gujarati</source>
-        <translation>Gujarati</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Oriya</source>
@@ -764,7 +914,7 @@ Check path and filename.</source>
     </message>
     <message>
         <source>Sinhala</source>
-        <translation>Sinhala</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Thai</source>
@@ -780,7 +930,7 @@ Check path and filename.</source>
     </message>
     <message>
         <source>Myanmar</source>
-        <translation>Myanmar</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Khmer</source>
@@ -788,15 +938,15 @@ Check path and filename.</source>
     </message>
     <message>
         <source>Han</source>
-        <translation>Han</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Hiragana</source>
-        <translation>Hiragana</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Katakana</source>
-        <translation>Katakana</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Hangul</source>
@@ -804,11 +954,11 @@ Check path and filename.</source>
     </message>
     <message>
         <source>Bopomofo</source>
-        <translation>Bopomofo</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Yi</source>
-        <translation>Yi</translation>
+        <translation></translation>
     </message>
     <message>
         <source>Ethiopic</source>
@@ -864,7 +1014,55 @@ Check path and filename.</source>
     </message>
     <message>
         <source>Unicode</source>
-        <translation>Уникод</translation>
+        <translation>Юникод</translation>
+    </message>
+    <message>
+        <source>SpacingModifiers</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>CombiningMarks</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Tagalog</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Hanunoo</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Buhid</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Tagbanwa</source>
+        <translation></translation>
+    </message>
+    <message>
+        <source>Katakana Half-Width Forms</source>
+        <translation>Katakana c половинными формами</translation>
+    </message>
+    <message>
+        <source>Han (Japanese)</source>
+        <translation>Han (Японский)</translation>
+    </message>
+    <message>
+        <source>Han (Simplified Chinese)</source>
+        <translation>Han (упрощенный Китайский)</translation>
+    </message>
+    <message>
+        <source>Han (Traditional Chinese)</source>
+        <translation>Han (традиционный Китайский)</translation>
+    </message>
+    <message>
+        <source>Han (Korean)</source>
+        <translation>Han (Корейский)</translation>
+    </message>
+    <message>
+        <source>Unknown Script</source>
+        <translation>Неизвестный скрипт</translation>
     </message>
 </context>
 <context>
@@ -895,7 +1093,7 @@ Check path and filename.</source>
     </message>
     <message>
         <source>&amp;Color</source>
-        <translation>&amp;Цвет</translation>
+        <translation type="obsolete">&amp;Цвет</translation>
     </message>
     <message>
         <source>Sample</source>
@@ -1031,7 +1229,7 @@ Check path and filename.</source>
     <name>QHeader</name>
     <message>
         <source>%1</source>
-        <translation>%1</translation>
+        <translation type="obsolete">%1</translation>
     </message>
 </context>
 <context>
@@ -1120,7 +1318,7 @@ Check path and filename.</source>
     </message>
     <message>
         <source>&amp;Redo</source>
-        <translation>&amp;Повторить</translation>
+        <translation>&amp;Восстановить</translation>
     </message>
     <message>
         <source>Cu&amp;t</source>
@@ -1201,35 +1399,35 @@ to
     <name>QMenuBar</name>
     <message>
         <source>About</source>
-        <translation>О программе</translation>
+        <translation type="obsolete">О программе</translation>
     </message>
     <message>
         <source>Config</source>
-        <translation>Конфигурация</translation>
+        <translation type="obsolete">Конфигурация</translation>
     </message>
     <message>
         <source>Preference</source>
-        <translation>Настройки</translation>
+        <translation type="obsolete">Параметры</translation>
     </message>
     <message>
         <source>Options</source>
-        <translation>Параметры</translation>
+        <translation type="obsolete">Параметры</translation>
     </message>
     <message>
         <source>Setting</source>
-        <translation>Настройки</translation>
+        <translation type="obsolete">Настройки</translation>
     </message>
     <message>
         <source>Setup</source>
-        <translation>Настройки</translation>
+        <translation type="obsolete">Настройки</translation>
     </message>
     <message>
         <source>Quit</source>
-        <translation>Выход</translation>
+        <translation type="obsolete">Выход</translation>
     </message>
     <message>
         <source>Exit</source>
-        <translation>Выход</translation>
+        <translation type="obsolete">Выход</translation>
     </message>
 </context>
 <context>
@@ -1264,7 +1462,19 @@ to
     </message>
     <message>
         <source>&lt;h3&gt;About Qt&lt;/h3&gt;&lt;p&gt;This program uses Qt version %1.&lt;/p&gt;&lt;p&gt;Qt is a C++ toolkit for multiplatform GUI &amp;amp; application development.&lt;/p&gt;&lt;p&gt;Qt provides single-source portability across MS&amp;nbsp;Windows, Mac&amp;nbsp;OS&amp;nbsp;X, Linux, and all major commercial Unix variants.&lt;br&gt;Qt is also available for embedded devices.&lt;/p&gt;&lt;p&gt;Qt is a Trolltech product. See &lt;tt&gt;http://www.trolltech.com/qt/&lt;/tt&gt; for more information.&lt;/p&gt;</source>
-        <translation>&lt;h3&gt;О Qt&lt;/h3&gt;&lt;p&gt;Эта программа использует Qt версии %1.&lt;/p&gt;&lt;p&gt;Qt - это C++ GUI инструментарий для разработки многоплатформенных приложений.&lt;/p&gt;&lt;p&gt;Qt обеспечивает переносимость на уровне исходного кода между MS&amp;nbsp;Windows, Mac&amp;nbsp;OS&amp;nbsp;X, Linux и всеми основными версиями Unix.&lt;br&gt;Qt также доступен для встраиваемых систем.&lt;/p&gt;&lt;p&gt;Qt является продуктом Trolltech. См. &lt;tt&gt;http://www.trolltech.com/qt/&lt;/tt&gt; для получения более подробной информации.&lt;/p&gt;</translation>
+        <translation>&lt;h3&gt;О Qt&lt;/h3&gt;&lt;p&gt;Эта программа использует Qt версии %1.&lt;/p&gt;&lt;p&gt;Qt - это C++ GUI инструментарий для разработки многоплатформенных приложений.&lt;/p&gt;&lt;p&gt;Qt обеспечивает переносимость на уровне исходного кода между MS&amp;nbsp;Windows, Mac&amp;nbsp;OS&amp;nbsp;X, Linux и всеми основными версиями Unix.&lt;br&gt;Qt также доступен для встраиваемых систем.&lt;/p&gt;&lt;p&gt;Qt является продуктом Trolltech. Посетите &lt;tt&gt;http://www.trolltech.com/qt/&lt;/tt&gt; для получения более подробной информации.&lt;/p&gt;</translation>
+    </message>
+    <message>
+        <source>Yes to &amp;All</source>
+        <translation>Да для &amp;всех</translation>
+    </message>
+    <message>
+        <source>N&amp;o to All</source>
+        <translation>&amp;Нет для всех</translation>
+    </message>
+    <message>
+        <source>About Qt</source>
+        <translation>О Qt</translation>
     </message>
 </context>
 <context>
@@ -1274,6 +1484,13 @@ to
         <translation>Операция прервана пользователем</translation>
     </message>
 </context>
+<context>
+    <name>QObject</name>
+    <message>
+        <source>On The Spot</source>
+        <translation></translation>
+    </message>
+</context>
 <context>
     <name>QPrintDialog</name>
     <message>
@@ -1378,11 +1595,11 @@ to
     </message>
     <message>
         <source>Portrait</source>
-        <translation>Портрет</translation>
+        <translation>Книжный</translation>
     </message>
     <message>
         <source>Landscape</source>
-        <translation>Альбом</translation>
+        <translation>Альбомный</translation>
     </message>
     <message>
         <source>A0 (841 x 1189 mm)</source>
@@ -1512,6 +1729,10 @@ to
         <source>PostScript Files (*.ps);;All Files (*)</source>
         <translation>Файлы PostScript (*.ps);;Все файлы (*)</translation>
     </message>
+    <message>
+        <source>Print selection</source>
+        <translation>Печатать выделенное</translation>
+    </message>
 </context>
 <context>
     <name>QProgressDialog</name>
@@ -1524,7 +1745,7 @@ to
     <name>QRegExp</name>
     <message>
         <source>no error occurred</source>
-        <translation>ошибки отсутствуют</translation>
+        <translation>ошибок не произошло</translation>
     </message>
     <message>
         <source>disabled feature used</source>
@@ -1532,15 +1753,15 @@ to
     </message>
     <message>
         <source>bad char class syntax</source>
-        <translation>bad char class syntax</translation>
+        <translation>ошибочный синтаксис класса  символа</translation>
     </message>
     <message>
         <source>bad lookahead syntax</source>
-        <translation>bad lookahead syntax</translation>
+        <translation>ошибочный синтаксис упреждающего чтения</translation>
     </message>
     <message>
         <source>bad repetition syntax</source>
-        <translation>bad repetition syntax</translation>
+        <translation>ошибочный синтаксис многократного чтения</translation>
     </message>
     <message>
         <source>invalid octal value</source>
@@ -1599,7 +1820,7 @@ to
     </message>
     <message>
         <source>Cancel your edits?</source>
-        <translation>Отменить изменения?</translation>
+        <translation>Не сохранять изменения?</translation>
     </message>
 </context>
 <context>
@@ -1633,11 +1854,11 @@ to
     </message>
     <message>
         <source>&amp;Redo</source>
-        <translation>&amp;Повторить</translation>
+        <translation>&amp;Восстановить</translation>
     </message>
     <message>
         <source>Cu&amp;t</source>
-        <translation>&amp;Вырезать</translation>
+        <translation>Вы&amp;резать</translation>
     </message>
     <message>
         <source>&amp;Copy</source>
@@ -1645,7 +1866,7 @@ to
     </message>
     <message>
         <source>&amp;Paste</source>
-        <translation>&amp;Вставить</translation>
+        <translation>В&amp;ставить</translation>
     </message>
     <message>
         <source>Clear</source>
@@ -1737,51 +1958,51 @@ to
     <name>QWSDecoration</name>
     <message>
         <source>&amp;Restore</source>
-        <translation>&amp;Восстановить</translation>
+        <translation type="obsolete">&amp;Восстановить</translation>
     </message>
     <message>
         <source>&amp;Move</source>
-        <translation>&amp;Переместить</translation>
+        <translation type="obsolete">&amp;Переместить</translation>
     </message>
     <message>
         <source>&amp;Size</source>
-        <translation>&amp;Размер</translation>
+        <translation type="obsolete">&amp;Размер</translation>
     </message>
     <message>
         <source>Mi&amp;nimize</source>
-        <translation>&amp;Свернуть</translation>
+        <translation type="obsolete">&amp;Свернуть</translation>
     </message>
     <message>
         <source>Ma&amp;ximize</source>
-        <translation>Р&amp;азвернуть</translation>
+        <translation type="obsolete">Р&amp;азвернуть</translation>
     </message>
     <message>
         <source>Close</source>
-        <translation>Закрыть</translation>
+        <translation type="obsolete">Закрыть</translation>
     </message>
     <message>
         <source>Windows</source>
-        <translation>Windows</translation>
+        <translation type="obsolete">Windows</translation>
     </message>
     <message>
         <source>KDE</source>
-        <translation>KDE</translation>
+        <translation type="obsolete">KDE</translation>
     </message>
     <message>
         <source>KDE2</source>
-        <translation>KDE2</translation>
+        <translation type="obsolete">KDE2</translation>
     </message>
     <message>
         <source>BeOS</source>
-        <translation>BeOS</translation>
+        <translation type="obsolete">BeOS</translation>
     </message>
     <message>
         <source>Hydro</source>
-        <translation>Hydro</translation>
+        <translation type="obsolete">Hydro</translation>
     </message>
     <message>
         <source>Default</source>
-        <translation>По умолчанию</translation>
+        <translation type="obsolete">По умолчанию</translation>
     </message>
 </context>
 <context>
@@ -1807,7 +2028,7 @@ to
     </message>
     <message>
         <source>&amp;Finish</source>
-        <translation>&amp;Финиш</translation>
+        <translation>&amp;Готово</translation>
     </message>
     <message>
         <source>&amp;Help</source>
@@ -1842,7 +2063,7 @@ to
     </message>
     <message>
         <source>Stay on &amp;Top</source>
-        <translation>Всегда &amp;наверху</translation>
+        <translation>Расположить &amp;поверх всех окон</translation>
     </message>
     <message>
         <source>Sh&amp;ade</source>
@@ -1889,15 +2110,15 @@ to
     </message>
     <message>
         <source>error occurred while parsing element</source>
-        <translation>в процессе грамматического разбора элемента произошла ошибка</translation>
+        <translation>в процессе синтаксического анализа элемента произошла ошибка</translation>
     </message>
     <message>
         <source>tag mismatch</source>
-        <translation>отсутствует тег</translation>
+        <translation>несоответствие тегов </translation>
     </message>
     <message>
         <source>error occurred while parsing content</source>
-        <translation>в процессе грамматического разбора произошла ошибка</translation>
+        <translation>в процессе синтаксического анализа произошла ошибка</translation>
     </message>
     <message>
         <source>unexpected character</source>
@@ -1925,43 +2146,43 @@ to
     </message>
     <message>
         <source>error occurred while parsing document type definition</source>
-        <translation>в процессе грамматического разбора типа документа произошла ошибка</translation>
+        <translation>в процессе синтаксического анализа типа документа произошла ошибка</translation>
     </message>
     <message>
         <source>letter is expected</source>
-        <translation>ожидался символ</translation>
+        <translation>ожидается символ</translation>
     </message>
     <message>
         <source>error occurred while parsing comment</source>
-        <translation>в процессе грамматического разбора комментария произошла ошибка</translation>
+        <translation>в процессе синтаксического анализа комментария произошла ошибка</translation>
     </message>
     <message>
         <source>error occurred while parsing reference</source>
-        <translation>в процессе грамматического разбора ссылки произошла ошибка</translation>
+        <translation>в процессе синтаксического анализа ссылки произошла ошибка</translation>
     </message>
     <message>
         <source>internal general entity reference not allowed in DTD</source>
-        <translation>internal general entity reference not allowed in DTD</translation>
+        <translation>ссылка на основные внутренние определения не разрешена в DTD</translation>
     </message>
     <message>
         <source>external parsed general entity reference not allowed in attribute value</source>
-        <translation>external parsed general entity reference not allowed in attribute value</translation>
+        <translation>ссылка на основные внутренние определения, обработанная извне, не разрешена в значении атрибута</translation>
     </message>
     <message>
         <source>external parsed general entity reference not allowed in DTD</source>
-        <translation>external parsed general entity reference not allowed in DTD</translation>
+        <translation>ссылка на основные внутренние определения, обработанная извне, не разрешена в DTD</translation>
     </message>
     <message>
         <source>unparsed entity reference in wrong context</source>
-        <translation>unparsed entity reference in wrong context</translation>
+        <translation>неанализированная ссылка на определение в неправильном контексте</translation>
     </message>
     <message>
         <source>recursive entities</source>
-        <translation>рекурсивные объекты</translation>
+        <translation>рекурсивные определения</translation>
     </message>
     <message>
         <source>error in the text declaration of an external entity</source>
-        <translation>error in the text declaration of an external entity</translation>
+        <translation>ошибка в тексте описания внешнего определения</translation>
     </message>
 </context>
 <context>
@@ -1972,7 +2193,7 @@ to
     </message>
     <message>
         <source>Redo</source>
-        <translation>Повторить</translation>
+        <translation>Восстановить</translation>
     </message>
     <message>
         <source>Cut</source>
